@@ -16,7 +16,7 @@ import pandas as pd
 from zebrazoom.code.vars import getGlobalVariables
 globalVariables = getGlobalVariables()
 
-from zebrazoom.mainZZ import mainZZ
+from zebrazoom.zebraZoomVideoAnalysis import ZebraZoomVideoAnalysis
 from zebrazoom.getTailExtremityFirstFrame import getTailExtremityFirstFrame
 import zebrazoom.code.paths as paths
 import zebrazoom.code.util as util
@@ -28,12 +28,13 @@ from PyQt5.QtWidgets import QAbstractItemView, QApplication, QCheckBox, QFileDia
 
 LARGE_FONT= ("Verdana", 12)
 
-def chooseVideoToAnalyze(self, justExtractParams, noValidationVideo, chooseFrames):
-    videoName, _ = QFileDialog.getOpenFileName(self.window, 'Select file', os.path.expanduser("~"))
+
+def chooseVideoToAnalyze(self, createValidationVideo, chooseFrames):
+    videoName, _ = QFileDialog.getOpenFileName(self.window, 'Select video', os.path.expanduser("~"), filter=f'Videos ({" ".join("*%s" % ext for ext in util.VIDEO_EXTENSIONS)});; All files (*.*)')
     if not videoName:
       return
     ZZargs = ([videoName],)
-    ZZkwargs = {'justExtractParams': justExtractParams, 'noValidationVideo': noValidationVideo}
+    ZZkwargs = {'createValidationVideo': createValidationVideo}
 
     if chooseFrames:
       def beginningAndEndChosen():
@@ -176,29 +177,6 @@ class _TrackingConfigurationsSelectionModel(QItemSelectionModel):
 
 
 class _VideoSelectionPage(QWidget):
-  _VIDEO_EXTENSIONS = {'.264', '.3g2', '.3gp', '.3gp2', '.3gpp', '.3gpp2', '.3mm', '.3p2', '.60d', '.787', '.89', '.aaf', '.aec', '.aep', '.aepx', '.aet', '.aetx', '.ajp',
-                       '.ale', '.am', '.amc', '.amv', '.amx', '.anim', '.aqt', '.arcut', '.arf', '.asf', '.asx', '.avb', '.avc', '.avd', '.avi', '.avp', '.avs', '.avs',
-                       '.avv', '.axm', '.bdm', '.bdmv', '.bdt2', '.bdt3', '.bik', '.bix', '.bmk', '.bnp', '.box', '.bs4', '.bsf', '.bvr', '.byu', '.camproj', '.camrec',
-                       '.camv', '.ced', '.cel', '.cine', '.cip', '.clpi', '.cmmp', '.cmmtpl', '.cmproj', '.cmrec', '.cpi', '.cst', '.cvc', '.cx3', '.d2v', '.d3v', '.dat',
-                       '.dav', '.dce', '.dck', '.dcr', '.dcr', '.ddat', '.dif', '.dir', '.divx', '.dlx', '.dmb', '.dmsd', '.dmsd3d', '.dmsm', '.dmsm3d', '.dmss', '.dmx', '.dnc',
-                       '.dpa', '.dpg', '.dream', '.dsy', '.dv', '.dv-avi', '.dv4', '.dvdmedia', '.dvr', '.dvr-ms', '.dvx', '.dxr', '.dzm', '.dzp', '.dzt', '.edl', '.evo', '.eye',
-                       '.ezt', '.f4p', '.f4v', '.fbr', '.fbr', '.fbz', '.fcp', '.fcproject', '.ffd', '.flc', '.flh', '.fli', '.flv', '.flx', '.gfp', '.gl', '.gom', '.grasp',
-                       '.gts', '.gvi', '.gvp', '.h264', '.hdmov', '.hkm', '.ifo', '.imovieproj', '.imovieproject', '.ircp', '.irf', '.ism', '.ismc', '.ismv', '.iva', '.ivf',
-                       '.ivr', '.ivs', '.izz', '.izzy', '.jss', '.jts', '.jtv', '.k3g', '.kmv', '.ktn', '.lrec', '.lsf', '.lsx', '.m15', '.m1pg', '.m1v', '.m21', '.m21', '.m2a',
-                       '.m2p', '.m2t', '.m2ts', '.m2v', '.m4e', '.m4u', '.m4v', '.m75', '.mani', '.meta', '.mgv', '.mj2', '.mjp', '.mjpg', '.mk3d', '.mkv', '.mmv', '.mnv',
-                       '.mob', '.mod', '.modd', '.moff', '.moi', '.moov', '.mov', '.movie', '.mp21', '.mp21', '.mp2v', '.mp4', '.mp4v', '.mpe', '.mpeg', '.mpeg1', '.mpeg4',
-                       '.mpf', '.mpg', '.mpg2', '.mpgindex', '.mpl', '.mpl', '.mpls', '.mpsub', '.mpv', '.mpv2', '.mqv', '.msdvd', '.mse', '.msh', '.mswmm', '.mts', '.mtv',
-                       '.mvb', '.mvc', '.mvd', '.mve', '.mvex', '.mvp', '.mvp', '.mvy', '.mxf', '.mxv', '.mys', '.ncor', '.nsv', '.nut', '.nuv', '.nvc', '.ogm', '.ogv', '.ogx',
-                       '.osp', '.otrkey', '.pac', '.par', '.pds', '.pgi', '.photoshow', '.piv', '.pjs', '.playlist', '.plproj', '.pmf', '.pmv', '.pns', '.ppj', '.prel', '.pro',
-                       '.prproj', '.prtl', '.psb', '.psh', '.pssd', '.pva', '.pvr', '.pxv', '.qt', '.qtch', '.qtindex', '.qtl', '.qtm', '.qtz', '.r3d', '.rcd', '.rcproject',
-                       '.rdb', '.rec', '.rm', '.rmd', '.rmd', '.rmp', '.rms', '.rmv', '.rmvb', '.roq', '.rp', '.rsx', '.rts', '.rts', '.rum', '.rv', '.rvid', '.rvl', '.sbk',
-                       '.sbt', '.scc', '.scm', '.scm', '.scn', '.screenflow', '.sec', '.sedprj', '.seq', '.sfd', '.sfvidcap', '.siv', '.smi', '.smi', '.smil', '.smk', '.sml',
-                       '.smv', '.spl', '.sqz', '.srt', '.ssf', '.ssm', '.stl', '.str', '.stx', '.svi', '.swf', '.swi', '.swt', '.tda3mt', '.tdx', '.thp', '.tivo', '.tix',
-                       '.tod', '.tp', '.tp0', '.tpd', '.tpr', '.trp', '.ts', '.tsp', '.ttxt', '.tvs', '.usf', '.usm', '.vc1', '.vcpf', '.vcr', '.vcv', '.vdo', '.vdr', '.vdx',
-                       '.veg','.vem', '.vep', '.vf', '.vft', '.vfw', '.vfz', '.vgz', '.vid', '.video', '.viewlet', '.viv', '.vivo', '.vlab', '.vob', '.vp3', '.vp6', '.vp7',
-                       '.vpj', '.vro', '.vs4', '.vse', '.vsp', '.w32', '.wcp', '.webm', '.wlmp', '.wm', '.wmd', '.wmmp', '.wmv', '.wmx', '.wot', '.wp3', '.wpl', '.wtv', '.wve',
-                       '.wvx', '.xej', '.xel', '.xesc', '.xfl', '.xlmv', '.xmv', '.xvid', '.y4m', '.yog', '.yuv', '.zeg', '.zm1', '.zm2', '.zm3', '.zmv'}
-
   def __init__(self, ZZkwargs):
     super().__init__()
     self._ZZkwargs = ZZkwargs
@@ -345,7 +323,7 @@ class _VideoSelectionPage(QWidget):
   def _addVideos(self):
     app = QApplication.instance()
     with self.__selectAddedRows():
-      self._table.model().addVideos(QFileDialog.getOpenFileNames(app.window, 'Select one or more videos', os.path.expanduser("~"), "Video files (%s)" % ' '.join('*%s' % ext for ext in self._VIDEO_EXTENSIONS))[0])
+      self._table.model().addVideos(QFileDialog.getOpenFileNames(app.window, 'Select one or more videos', os.path.expanduser("~"), "Video files (%s)" % ' '.join('*%s' % ext for ext in util.VIDEO_EXTENSIONS))[0])
 
   def _addFolder(self):
     app = QApplication.instance()
@@ -353,7 +331,7 @@ class _VideoSelectionPage(QWidget):
     if selectedFolder is None:
       return
     with self.__selectAddedRows():
-      return self._table.model().addVideos([os.path.normpath(os.path.join(root, f)) for root, _dirs, files in os.walk(selectedFolder) for f in files if os.path.splitext(f)[1] in self._VIDEO_EXTENSIONS])
+      return self._table.model().addVideos([os.path.normpath(os.path.join(root, f)) for root, _dirs, files in os.walk(selectedFolder) for f in files if os.path.splitext(f)[1] in util.VIDEO_EXTENSIONS])
 
   def _addMultipleFolders(self):
     app = QApplication.instance()
@@ -374,10 +352,10 @@ class _VideoSelectionPage(QWidget):
       return None
     with self.__selectAddedRows():
       self._table.model().addVideos([os.path.normpath(os.path.join(root, f)) for path in dialog.selectedFiles()
-                                     for root, _dirs, files in os.walk(path) for f in files if os.path.splitext(f)[1] in self._VIDEO_EXTENSIONS])
+                                     for root, _dirs, files in os.walk(path) for f in files if os.path.splitext(f)[1] in util.VIDEO_EXTENSIONS])
 
   def _updateParallelTracking(self):
-    if self._ZZkwargs.get('sbatchMode', False):
+    if self._ZZkwargs.get('sbatchMode', False) or self._ZZkwargs.get('findMultipleROIs', False) or self._ZZkwargs.get('headEmbedded', False):
       return
     videos, configs = self._table.model().getData()
     enabled = bool(configs)
@@ -387,7 +365,7 @@ class _VideoSelectionPage(QWidget):
         break
       with open(config) as f:
         cfg = json.load(f)
-      if not (cfg.get('fasterMultiprocessing', False) or cfg.get('headEmbeded', False)):
+      if not (cfg.get('fasterMultiprocessing', False) or cfg.get('headEmbeded', False) or (cfg.get('trackingImplementation') == "fastFishTracking.tracking")):
         enabled = False
         break
     self._parallelTrackingCheckbox.setVisible(enabled)
@@ -450,7 +428,7 @@ class _VideoSelectionPage(QWidget):
     if errors:
       error = QMessageBox(app.window)
       error.setIcon(QMessageBox.Icon.Critical)
-      error.setWindowTitle("Specification contains some error")
+      error.setWindowTitle("Specification contains some errors")
       error.setText("Cannot run tracking becase the specified video and config combinations contain some errors:")
       error.setDetailedText("\n".join(errors))
       textEdit = error.findChild(QTextEdit)
@@ -470,10 +448,12 @@ class _VideoSelectionPage(QWidget):
         break
       with open(config) as f:
         cfg = json.load(f)
+      videoName = os.path.splitext(os.path.basename(video))[0]
+      inputsFolder = os.path.join(app.ZZoutputLocation, '.ZebraZoomVideoInputs', videoName)
       if (cfg.get("multipleROIsDefinedDuringExecution", False) or cfg.get("groupOfMultipleSameSizeAndShapeEquallySpacedWells", False)) and \
-          not os.path.exists(os.path.join(app.ZZoutputLocation, os.path.splitext(os.path.basename(video))[0], 'intermediaryWellPositionReloadNoMatterWhat.txt')):
+          not os.path.exists(os.path.join(inputsFolder, 'intermediaryWellPositionReloadNoMatterWhat.txt')):
         defineWellsVideos.append((video, config))
-      if cfg.get("headEmbeded", False) and (not os.path.exists('%s.csv' % video) or not os.path.exists('%sHP.csv' % video)):
+      if cfg.get("headEmbeded", False) and (not os.path.exists(os.path.join(inputsFolder, f'{videoName}.csv')) or not os.path.exists(os.path.join(inputsFolder, f'{videoName}HP.csv'))):
         headEmbeddedVideos.append((video, config))
     if defineWellsVideos:
       text = 'Some of the videos do not have the regions defined. Would you like to define them before running tracking?'
@@ -496,10 +476,12 @@ class _VideoSelectionPage(QWidget):
         ZZkwargs.update({'headEmbedded': True, 'processes': 1})
         launchZebraZoom(*zip(*headEmbeddedVideos), **ZZkwargs)
     app.show_frame("Patience")
-    app.window.centralWidget().layout().currentWidget().setArgs((videos, configs), self._ZZkwargs)
+    kwargs = self._ZZkwargs.copy()
+    kwargs['showValidationVideoWidget'] = not (kwargs.get('sbatchMode', False) or kwargs.get('findMultipleROIs', False) or kwargs.get('headEmbedded', False))
+    app.window.centralWidget().layout().currentWidget().setArgs((videos, configs), kwargs)
 
 
-def _showVideoSelectionPage(ZZkwargs):
+def _showTemporaryPage(ZZkwargs):
   app = QApplication.instance()
   layout = app.window.centralWidget().layout()
   page = _VideoSelectionPage(ZZkwargs)
@@ -511,19 +493,19 @@ def _showVideoSelectionPage(ZZkwargs):
   layout.currentChanged.connect(cleanup)
 
 
-def chooseFolderToAnalyze(self, justExtractParams, noValidationVideo, sbatchMode):
-  ZZkwargs = {'justExtractParams': justExtractParams, 'noValidationVideo': noValidationVideo, 'sbatchMode': sbatchMode}
-  _showVideoSelectionPage(ZZkwargs)
+def chooseFolderToAnalyze(self, sbatchMode):
+  ZZkwargs = {'sbatchMode': sbatchMode}
+  _showTemporaryPage(ZZkwargs)
 
 
 def chooseFolderForTailExtremityHE(self):
   ZZkwargs = {'headEmbedded': True}
-  _showVideoSelectionPage(ZZkwargs)
+  _showTemporaryPage(ZZkwargs)
 
 
 def chooseFolderForMultipleROIs(self, askCoordinatesForAll):
   ZZkwargs = {'findMultipleROIs': True, 'askCoordinatesForAll': askCoordinatesForAll}
-  _showVideoSelectionPage(ZZkwargs)
+  _showTemporaryPage(ZZkwargs)
 
 
 def chooseConfigFile(ZZargs, ZZkwargs):
@@ -539,20 +521,18 @@ def chooseConfigFile(ZZargs, ZZkwargs):
     launchZebraZoom(*ZZargs, **ZZkwargs)
 
 
-def _runTracking(args, justExtractParams, noValidationVideo, ZZoutputLocation):
+def _runTracking(args, createValidationVideo, ZZoutputLocation):
   videoPath, config = args
   path        = os.path.dirname(videoPath)
   nameWithExt = os.path.basename(videoPath)
   name        = os.path.splitext(nameWithExt)[0]
   videoExt    = os.path.splitext(nameWithExt)[1][1:]
 
-  tabParams = ["mainZZ", path, name, videoExt, config, "freqAlgoPosFollow", 100, "outputFolder", ZZoutputLocation]
-  if justExtractParams:
-    tabParams.extend(["reloadWellPositions", 1, "reloadBackground", 1, "debugPauseBetweenTrackAndParamExtract", "justExtractParamFromPreviousTrackData"])
-  if noValidationVideo:
-    tabParams.extend(["createValidationVideo", 0])
+  tabParams = ["zebraZoomVideoAnalysis", path, name, videoExt, config, "freqAlgoPosFollow", 100, "popUpAlgoFollow", 0, "outputFolder", ZZoutputLocation]
+  if createValidationVideo is not None:
+    tabParams.extend(['createValidationVideo', createValidationVideo])
   try:
-    mainZZ(path, name, videoExt, config, tabParams)
+    ZebraZoomVideoAnalysis(path, name, videoExt, config, tabParams).run()
   except ValueError:
     print("moving on to the next video for ROIs identification")
   except NameError:
@@ -560,7 +540,7 @@ def _runTracking(args, justExtractParams, noValidationVideo, ZZoutputLocation):
 
 
 @util.showInProgressPage('Tracking')
-def launchZebraZoom(videos, configs, headEmbedded=False, sbatchMode=False, justExtractParams=False, noValidationVideo=False, findMultipleROIs=False,
+def launchZebraZoom(videos, configs, headEmbedded=False, sbatchMode=False, createValidationVideo=None, findMultipleROIs=False,
                     askCoordinatesForAll=True, firstFrame=None, lastFrame=None, backgroundExtractionForceUseAllVideoFrames=None, processes=1):
   app = QApplication.instance()
 
@@ -568,25 +548,28 @@ def launchZebraZoom(videos, configs, headEmbedded=False, sbatchMode=False, justE
     commandsFile = open(os.path.join(paths.getRootDataFolder(), "commands.txt"), "w", newline='\n')
     nbVideosToLaunch = 0
 
+  videosGenerator = None
   if len(videos) > 1:
     if not askCoordinatesForAll:
       videosGenerator = iter(videos)
       videos = [next(videosGenerator)]
   else:
     videoPath = videos[0]
-    if (os.path.exists(videoPath + 'HP.csv') or os.path.exists(videoPath + '.csv')) and \
+    videoName = os.path.splitext(os.path.basename(videoPath))[0]
+    inputsFolder = os.path.join(app.ZZoutputLocation, '.ZebraZoomVideoInputs', videoName)
+    if (os.path.exists(os.path.join(inputsFolder, f'{videoName}HP.csv')) or os.path.exists(os.path.join(inputsFolder, f'{videoName}.csv'))) and \
         QMessageBox.question(app.window, "Previously stored coordinates found", "Do you want to use the previously stored coordinates?",
                              defaultButton=QMessageBox.StandardButton.Yes) != QMessageBox.StandardButton.Yes:
-      if os.path.exists(videoPath + 'HP.csv'):
-        os.remove(videoPath + 'HP.csv')
-      if os.path.exists(videoPath + '.csv'):
-        os.remove(videoPath + '.csv')
+      if os.path.exists(os.path.join(inputsFolder, f'{videoName}HP.csv')):
+        os.remove(os.path.join(inputsFolder, f'{videoName}HP.csv'))
+      if os.path.exists(os.path.join(inputsFolder, f'{videoName}.csv')):
+        os.remove(os.path.join(inputsFolder, f'{videoName}.csv'))
 
   print("allVideos:", videos)
 
   if processes > 1 and len(videos) > 1 and not sbatchMode:
     with Pool(min(processes, len(videos))) as pool:
-      pool.map(partial(_runTracking, justExtractParams=justExtractParams, noValidationVideo=noValidationVideo, ZZoutputLocation=app.ZZoutputLocation), zip(videos, configs))
+      pool.map(partial(_runTracking, createValidationVideo=createValidationVideo, ZZoutputLocation=app.ZZoutputLocation), zip(videos, configs))
   else:
     for videoPath, config in zip(videos, configs):
 
@@ -597,19 +580,17 @@ def launchZebraZoom(videos, configs, headEmbedded=False, sbatchMode=False, justE
 
       if not headEmbedded:
         if len(videos) == 1:
-          tabParams = ["mainZZ", path, name, videoExt, config, "freqAlgoPosFollow", 100, "popUpAlgoFollow", 1, "outputFolder", app.ZZoutputLocation]
+          tabParams = ["zebraZoomVideoAnalysis", path, name, videoExt, config, "freqAlgoPosFollow", 100, "popUpAlgoFollow", 1, "outputFolder", app.ZZoutputLocation]
         else:
-          tabParams = ["mainZZ", path, name, videoExt, config, "freqAlgoPosFollow", 100, "outputFolder", app.ZZoutputLocation]
+          tabParams = ["zebraZoomVideoAnalysis", path, name, videoExt, config, "freqAlgoPosFollow", 100, "outputFolder", app.ZZoutputLocation]
         if backgroundExtractionForceUseAllVideoFrames is not None:
           tabParams.extend(["backgroundExtractionForceUseAllVideoFrames", int(backgroundExtractionForceUseAllVideoFrames)])
         if firstFrame is not None:
           tabParams.extend(["firstFrame", firstFrame])
         if lastFrame is not None:
           tabParams.extend(["lastFrame", lastFrame])
-        if justExtractParams:
-          tabParams = tabParams + ["reloadWellPositions", 1, "reloadBackground", 1, "debugPauseBetweenTrackAndParamExtract", "justExtractParamFromPreviousTrackData"]
-        if noValidationVideo:
-            tabParams = tabParams + ["createValidationVideo", 0]
+        if createValidationVideo is not None:
+          tabParams.extend(['createValidationVideo', createValidationVideo])
         if findMultipleROIs:
           tabParams = tabParams + ["exitAfterWellsDetection", 1, "saveWellPositionsToBeReloadedNoMatterWhat", 1]
         try:
@@ -617,12 +598,10 @@ def launchZebraZoom(videos, configs, headEmbedded=False, sbatchMode=False, justE
             commandsFile.write('python -m zebrazoom ' + ' '.join(tabParams[1:4]) + ' configFiles/%s\n' % os.path.basename(config))
             nbVideosToLaunch = nbVideosToLaunch + 1
           else:
-            mainZZ(path, name, videoExt, config, tabParams)
+            ZebraZoomVideoAnalysis(path, name, videoExt, config, tabParams).run()
         except ValueError:
           print("moving on to the next video for ROIs identification")
-        except NameError:
-          app.show_frame("Error")
-          return
+
       else:
         tabParams = ["outputFolder", app.ZZoutputLocation]
         if backgroundExtractionForceUseAllVideoFrames is not None:
@@ -633,16 +612,15 @@ def launchZebraZoom(videos, configs, headEmbedded=False, sbatchMode=False, justE
           tabParams.extend(["lastFrame", lastFrame])
         getTailExtremityFirstFrame(path, name, videoExt, config, tabParams)
 
-  if findMultipleROIs and not askCoordinatesForAll:
-    coordinatesFile = os.path.join(app.ZZoutputLocation, os.path.splitext(os.path.basename(videos[0]))[0], 'intermediaryWellPositionReloadNoMatterWhat.txt')
-    configUsedFile = os.path.join(app.ZZoutputLocation, os.path.splitext(os.path.basename(videos[0]))[0], 'intermediaryWellPositionReloadNoMatterWhat.txt')
-    rotationFile = os.path.join(app.ZZoutputLocation, os.path.splitext(os.path.basename(videos[0]))[0], 'rotationAngle.txt')
+  if videosGenerator is not None:
+    videoInputsFolder = os.path.join(app.ZZoutputLocation, '.ZebraZoomVideoInputs', os.path.splitext(os.path.basename(videos[0]))[0])
+    coordinatesFile = os.path.join(videoInputsFolder, 'intermediaryWellPositionReloadNoMatterWhat.txt')
+    rotationFile = os.path.join(videoInputsFolder, 'rotationAngle.txt')
     for video in videosGenerator:
-      folderPath = os.path.join(app.ZZoutputLocation, os.path.splitext(os.path.basename(video))[0])
+      folderPath = os.path.join(app.ZZoutputLocation, '.ZebraZoomVideoInputs', os.path.splitext(os.path.basename(video))[0])
       if not os.path.exists(folderPath):
         os.makedirs(folderPath)
       shutil.copy2(coordinatesFile, os.path.join(folderPath, 'intermediaryWellPositionReloadNoMatterWhat.txt'))
-      shutil.copy2(configUsedFile, os.path.join(folderPath, 'configUsed.json'))
       if os.path.exists(rotationFile):
         shutil.copy2(rotationFile, os.path.join(folderPath, 'rotationAngle.txt'))
 
@@ -652,7 +630,12 @@ def launchZebraZoom(videos, configs, headEmbedded=False, sbatchMode=False, justE
 
     with open(configs[0]) as f:
       jsonFile = json.load(f)
-    nbWells = jsonFile["nbWells"]
+    nbWells = jsonFile.get("nbWells")
+    if nbWells is None:
+      if jsonFile.get('groupOfMultipleSameSizeAndShapeEquallySpacedWells', False):
+        nbWells = jsonFile["nbWellsPerRows"] * jsonFile["nbRowsOfWells"]
+      else:
+        nbWells = 0 if jsonFile.get('noWellDetection', False) else 1
     if nbWells > 24:
       nbWells = 24
 
