@@ -1,13 +1,12 @@
 import numpy as np
-from zebrazoom.mainZZ import mainZZ
+from zebrazoom.zebraZoomVideoAnalysis import ZebraZoomVideoAnalysis
 import os
 import pickle
 import cv2
 import zebrazoom.videoFormatConversion.zzVideoReading as zzVideoReading
 from zebrazoom.code.GUI.adjustParameterInsideAlgo import adjustParamInsideAlgoPage, adjustBoutDetectionOnlyPage, adjustParamInsideAlgoFreelySwimPage, adjustParamInsideAlgoFreelySwimAutomaticParametersPage
 
-from PyQt5.QtCore import Qt
-from PyQt5.QtWidgets import QApplication, QDialog, QLabel, QPushButton, QVBoxLayout
+from PyQt5.QtWidgets import QApplication
 
 import zebrazoom.code.paths as paths
 import zebrazoom.code.util as util
@@ -22,7 +21,7 @@ def getMainArguments(self):
   videoExt     = nameWithExtL.pop()
   videoName    = '.'.join(nameWithExtL)
   configFile   = self.configFile
-  argv         = []
+  argv = ["outputFolder", self.ZZoutputLocation]
   return [pathToVideo, videoName, videoExt, configFile, argv]
 
 
@@ -57,11 +56,19 @@ def prepareConfigFileForParamsAdjustements(configFile, wellNumber, firstFrame, v
   return initialFirstFrameValue, initialLastFrameValue
 
 
-def detectBouts(self, controller, wellNumber, firstFrame, adjustOnWholeVideo):
+def detectBouts(self, controller, wellNumber, firstFrame, adjustOnWholeVideo, reloadWellPositions=True):
   [pathToVideo, videoName, videoExt, configFile, argv] = getMainArguments(self)
 
   initialFirstFrameValue, initialLastFrameValue = prepareConfigFileForParamsAdjustements(configFile, wellNumber, firstFrame, self.videoToCreateConfigFileFor, adjustOnWholeVideo)
 
+  if configFile.get('trackingImplementation') == 'fastFishTracking.tracking':
+    configFile['detectMovementWithRawVideoInsideTracking'] = 1
+
+  paramsToRemove = ["useFirstFrameAsBackground", "updateBackgroundAtInterval", "detectMovementWithRawVideoInsideTracking"]
+  if configFile.get("fasterMultiprocessing", 0) == 2:
+    paramsToRemove.append("fasterMultiprocessing")
+  temporarilyRemovedParams = {param: configFile.pop(param, None) for param in paramsToRemove}
+
   configFile["noBoutsDetection"] = 0
   if "trackTail" in configFile:
     trackTailOriginalValue = configFile["trackTail"]
@@ -69,7 +76,7 @@ def detectBouts(self, controller, wellNumber, firstFrame, adjustOnWholeVideo):
     trackTailOriginalValue = 1
   configFile["trackTail"]                   = 0
   configFile["adjustDetectMovWithRawVideo"] = 1
-  configFile["reloadWellPositions"]         = 1
+  configFile["reloadWellPositions"]         = int(reloadWellPositions)
   if "freqAlgoPosFollow" in configFile:
     freqAlgoPosFollowInitial = configFile["freqAlgoPosFollow"]
   else:
@@ -85,20 +92,27 @@ def detectBouts(self, controller, wellNumber, firstFrame, adjustOnWholeVideo):
   app = QApplication.instance()
   with app.busyCursor():
     try:
+      storeH5 = configFile.get('storeH5')
+      configFile['storeH5'] = 1
       if "lastFrame" in configFile and "firstFrame" in configFile and configFile["lastFrame"] < configFile["firstFrame"]:
         del configFile["lastFrame"]
-      mainZZ(pathToVideo, videoName, videoExt, configFile, argv)
+      ZebraZoomVideoAnalysis(pathToVideo, videoName, videoExt, configFile, argv).run()
     except ValueError:
       newhyperparameters = pickle.load(open(os.path.join(paths.getRootDataFolder(), 'newhyperparameters'), 'rb'))
       for index in newhyperparameters:
         configFile[index] = newhyperparameters[index]
     except NameError:
       print("Configuration file parameters changes discarded.")
+    finally:
+      if storeH5 is not None:
+        configFile['storeH5'] = storeH5
+      else:
+        del configFile['storeH5']
 
-  configFile["onlyTrackThisOneWell"]        = -1
+  del configFile["onlyTrackThisOneWell"]
   configFile["trackTail"]                   = trackTailOriginalValue
-  configFile["adjustDetectMovWithRawVideo"] = 0
-  configFile["reloadWellPositions"]         = 0
+  del configFile["adjustDetectMovWithRawVideo"]
+  del configFile["reloadWellPositions"]
 
   if initialLastFrameValue == -1:
     if 'firstFrame' in configFile:
@@ -115,7 +129,9 @@ def detectBouts(self, controller, wellNumber, firstFrame, adjustOnWholeVideo):
     configFile["freqAlgoPosFollow"] = freqAlgoPosFollowInitial
   
   configFile["reloadBackground"] = 0
-  
+
+  configFile.update({param: value for param, value in temporarilyRemovedParams.items() if value is not None})
+
   self.configFile = configFile
 
 
@@ -128,15 +144,22 @@ def adjustHeadEmbededTracking(self, controller, wellNumber, firstFrame, adjustOn
   configFile["adjustHeadEmbededTracking"] = 1
 
   try:
+    storeH5 = configFile.get('storeH5')
+    configFile['storeH5'] = 1
     if "lastFrame" in configFile and "firstFrame" in configFile and configFile["lastFrame"] < configFile["firstFrame"]:
       del configFile["lastFrame"]
-    mainZZ(pathToVideo, videoName, videoExt, configFile, argv)
+    ZebraZoomVideoAnalysis(pathToVideo, videoName, videoExt, configFile, argv).run()
   except ValueError:
     newhyperparameters = pickle.load(open(os.path.join(paths.getRootDataFolder(), 'newhyperparameters'), 'rb'))
     for index in newhyperparameters:
       configFile[index] = newhyperparameters[index]
   except NameError:
     print("Configuration file parameters changes discarded.")
+  finally:
+    if storeH5 is not None:
+      configFile['storeH5'] = storeH5
+    else:
+      del configFile['storeH5']
 
   configFile["onlyTrackThisOneWell"]      = -1
   configFile["adjustHeadEmbededTracking"] = 0
@@ -164,15 +187,22 @@ def adjustFreelySwimTracking(self, controller, wellNumber, firstFrame, adjustOnW
   configFile["adjustFreelySwimTracking"] = 1
 
   try:
+    storeH5 = configFile.get('storeH5')
+    configFile['storeH5'] = 1
     if "lastFrame" in configFile and "firstFrame" in configFile and configFile["lastFrame"] < configFile["firstFrame"]:
       del configFile["lastFrame"]
-    mainZZ(pathToVideo, videoName, videoExt, configFile, argv)
+    ZebraZoomVideoAnalysis(pathToVideo, videoName, videoExt, configFile, argv).run()
   except ValueError:
     newhyperparameters = pickle.load(open(os.path.join(paths.getRootDataFolder(), 'newhyperparameters'), 'rb'))
     for index in newhyperparameters:
       configFile[index] = newhyperparameters[index]
   except NameError:
     print("Configuration file parameters changes discarded.")
+  finally:
+    if storeH5 is not None:
+      configFile['storeH5'] = storeH5
+    else:
+      del configFile['storeH5']
 
   configFile["onlyTrackThisOneWell"]      = -1
   configFile["adjustFreelySwimTracking"] = 0
@@ -191,7 +221,7 @@ def adjustFreelySwimTracking(self, controller, wellNumber, firstFrame, adjustOnW
   self.configFile = configFile
 
 
-def _adjustFastFreelySwimTracking(self, controller, oldFirstFrame):
+def _adjustFastFreelySwimTracking(self, controller, oldFirstFrame, detectBouts):
   wellNumber = 0
   adjustOnWholeVideo = True
 
@@ -209,15 +239,22 @@ def _adjustFastFreelySwimTracking(self, controller, oldFirstFrame):
   configFile["reloadBackground"] = 1
 
   try:
+    storeH5 = configFile.get('storeH5')
+    configFile['storeH5'] = 1
     if "lastFrame" in configFile and "firstFrame" in configFile and configFile["lastFrame"] < configFile["firstFrame"]:
       del configFile["lastFrame"]
-    mainZZ(pathToVideo, videoName, videoExt, configFile, argv)
+    ZebraZoomVideoAnalysis(pathToVideo, videoName, videoExt, configFile, argv).run()
   except ValueError:
     newhyperparameters = pickle.load(open(os.path.join(paths.getRootDataFolder(), 'newhyperparameters'), 'rb'))
     for index in newhyperparameters:
       configFile[index] = newhyperparameters[index]
   except NameError:
     print("Configuration file parameters changes discarded.")
+  finally:
+    if storeH5 is not None:
+      configFile['storeH5'] = storeH5
+    else:
+      del configFile['storeH5']
 
   del configFile["reloadBackground"]
   del configFile["reloadWellPositions"]
@@ -241,13 +278,16 @@ def _adjustFastFreelySwimTracking(self, controller, oldFirstFrame):
     self.configFile["nbAnimalsPerWell"] = 1
     self.configFile["minPixelDiffForBackExtract"] = configFile["minPixelDiffForBackExtract"]
 
-  util.addToHistory(controller.show_frame)("FinishConfig")
+  if detectBouts:
+    util.addToHistory(controller.calculateBackgroundFreelySwim)(controller, 0, boutDetectionsOnly=True)
+  else:
+    util.addToHistory(controller.show_frame)("FinishConfig")
 
 
-def adjustFastFreelySwimTracking(self, controller):
+def adjustFastFreelySwimTracking(self, controller, detectBouts):
   oldFirstFrame = self.configFile.get("firstFrame")
   util.chooseBeginningPage(QApplication.instance(), self.videoToCreateConfigFileFor, "Choose where you want to start the procedure to adjust parameters.", "Ok, I want the procedure to start at this frame.",
-                           lambda: _adjustFastFreelySwimTracking(self, controller, oldFirstFrame))
+                           lambda: _adjustFastFreelySwimTracking(self, controller, oldFirstFrame, detectBouts))
 
 
 def adjustFreelySwimTrackingAutomaticParameters(self, controller, wellNumber, firstFrame, adjustOnWholeVideo):
@@ -260,15 +300,22 @@ def adjustFreelySwimTrackingAutomaticParameters(self, controller, wellNumber, fi
   configFile["adjustFreelySwimTrackingAutomaticParameters"] = 1
 
   try:
+    storeH5 = configFile.get('storeH5')
+    configFile['storeH5'] = 1
     if "lastFrame" in configFile and "firstFrame" in configFile and configFile["lastFrame"] < configFile["firstFrame"]:
       del configFile["lastFrame"]
-    mainZZ(pathToVideo, videoName, videoExt, configFile, argv)
+    ZebraZoomVideoAnalysis(pathToVideo, videoName, videoExt, configFile, argv).run()
   except ValueError:
     newhyperparameters = pickle.load(open(os.path.join(paths.getRootDataFolder(), 'newhyperparameters'), 'rb'))
     for index in newhyperparameters:
       configFile[index] = newhyperparameters[index]
   except NameError:
     print("Configuration file parameters changes discarded.")
+  finally:
+    if storeH5 is not None:
+      configFile['storeH5'] = storeH5
+    else:
+      del configFile['storeH5']
 
   del configFile["onlyTrackThisOneWell"]
   del configFile["adjustFreelySwimTrackingAutomaticParameters"]
@@ -288,7 +335,7 @@ def adjustFreelySwimTrackingAutomaticParameters(self, controller, wellNumber, fi
   self.configFile = configFile
 
 
-def calculateBackground(self, controller, nbImagesForBackgroundCalculation, useNext=True):
+def calculateBackground(self, controller, nbImagesForBackgroundCalculation, useNext=True, wellPositions=None, wellShape=None, frameIdx=None):
 
   [pathToVideo, videoName, videoExt, configFile, argv] = getMainArguments(self)
 
@@ -299,44 +346,69 @@ def calculateBackground(self, controller, nbImagesForBackgroundCalculation, useN
   if int(nbImagesForBackgroundCalculation):
     configFile["nbImagesForBackgroundCalculation"] = int(nbImagesForBackgroundCalculation)
 
+  storeH5 = configFile.get('storeH5')
+  configFile['storeH5'] = 1
+  if "lastFrame" in configFile and "firstFrame" in configFile and configFile["lastFrame"] < configFile["firstFrame"]:
+    del configFile["lastFrame"]
+
   app = QApplication.instance()
   app.wellPositions = []
   app.background = []
   with app.busyCursor():
+    if wellPositions is not None:
+      zzAnalysis = ZebraZoomVideoAnalysis(pathToVideo, videoName, videoExt, configFile, ['outputFolder', app.ZZoutputLocation])
+      zzAnalysis.storeConfigUsed(configFile)
+      zzAnalysis.storeWellPositions(wellPositions)
     try:
-      if "lastFrame" in configFile and "firstFrame" in configFile and configFile["lastFrame"] < configFile["firstFrame"]:
-        del configFile["lastFrame"]
-      mainZZ(pathToVideo, videoName, videoExt, configFile, argv)
+      ZebraZoomVideoAnalysis(pathToVideo, videoName, videoExt, configFile, argv).run()
     except ValueError:
       configFile["exitAfterBackgroundExtraction"] = 0
+    finally:
+      if storeH5 is not None:
+        configFile['storeH5'] = storeH5
+      else:
+        del configFile['storeH5']
 
   configFile["exitAfterBackgroundExtraction"]   = 0
   configFile["headEmbededRemoveBack"]           = 0
   configFile["debugExtractBack"]                = 0
 
-  adjustParamInsideAlgoPage(useNext=useNext)
+  adjustParamInsideAlgoPage(useNext=useNext, frameIdx=frameIdx)
 
 
-def calculateBackgroundFreelySwim(self, controller, nbImagesForBackgroundCalculation, morePreciseFastScreen=False, automaticParameters=False, boutDetectionsOnly=False, useNext=True):
+def calculateBackgroundFreelySwim(self, controller, nbImagesForBackgroundCalculation, morePreciseFastScreen=False, automaticParameters=False, boutDetectionsOnly=False, useNext=True, nextCb=None, reloadWellPositions=False, wellPositions=None, wellShape=None, wellIdx=0, frameIdx=None):
 
   [pathToVideo, videoName, videoExt, configFile, argv] = getMainArguments(self)
 
   configFile["exitAfterBackgroundExtraction"] = 1
   configFile["debugExtractBack"]              = 1
-  configFile["debugFindWells"]                = 1
+  configFile["debugFindWells"]                = int(wellPositions is None)
+  configFile["reloadWellPositions"] = int(reloadWellPositions)
 
   if int(nbImagesForBackgroundCalculation):
     configFile["nbImagesForBackgroundCalculation"] = int(nbImagesForBackgroundCalculation)
 
+  storeH5 = configFile.get('storeH5')
+  configFile['storeH5'] = 1
+  if "lastFrame" in configFile and "firstFrame" in configFile and configFile["lastFrame"] < configFile["firstFrame"]:
+    del configFile["lastFrame"]
+
   app = QApplication.instance()
   app.wellPositions = []
   with app.busyCursor():
+    if wellPositions is not None:
+      zzAnalysis = ZebraZoomVideoAnalysis(pathToVideo, videoName, videoExt, configFile, ['outputFolder', app.ZZoutputLocation])
+      zzAnalysis.storeConfigUsed(configFile)
+      zzAnalysis.storeWellPositions(wellPositions)
     try:
-      if "lastFrame" in configFile and "firstFrame" in configFile and configFile["lastFrame"] < configFile["firstFrame"]:
-        del configFile["lastFrame"]
-      mainZZ(pathToVideo, videoName, videoExt, configFile, argv)
+      ZebraZoomVideoAnalysis(pathToVideo, videoName, videoExt, configFile, argv).run()
     except ValueError:
       configFile["exitAfterBackgroundExtraction"] = 0
+    finally:
+      if storeH5 is not None:
+        configFile['storeH5'] = storeH5
+      else:
+        del configFile['storeH5']
 
   del configFile["exitAfterBackgroundExtraction"] #  = 0
   del configFile["debugExtractBack"]              #  = 0
@@ -354,14 +426,11 @@ def calculateBackgroundFreelySwim(self, controller, nbImagesForBackgroundCalcula
   if "reloadWellPositions" in configFile:
     del configFile["reloadWellPositions"]
 
-
-  if boutDetectionsOnly:
-    adjustBoutDetectionOnlyPage(useNext=useNext)
+  if morePreciseFastScreen:
+    adjustFastFreelySwimTracking(self, controller, boutDetectionsOnly)
+  elif boutDetectionsOnly:
+    adjustBoutDetectionOnlyPage(useNext=useNext, nextCb=nextCb)
+  elif automaticParameters:
+    adjustParamInsideAlgoFreelySwimAutomaticParametersPage(useNext=useNext, wellIdx=wellIdx, frameIdx=frameIdx)
   else:
-    if automaticParameters:
-      adjustParamInsideAlgoFreelySwimAutomaticParametersPage(useNext=useNext)
-    else:
-      if morePreciseFastScreen:
-        adjustFastFreelySwimTracking(self, controller)
-      else:
-        adjustParamInsideAlgoFreelySwimPage(useNext=useNext)
+    adjustParamInsideAlgoFreelySwimPage(useNext=useNext)
