@@ -8,15 +8,99 @@ import webbrowser
 MAX_INT32 = 2 ** 31 - 1
 
 
-def _createWidget(layout, status, values, info, name, widgets, hasCheckbox):
-  from PyQt5.QtCore import Qt
+def __calculateSteps(maxDepth, numberOfSteps):
+  steps = [maxDepth / 1.8, maxDepth / 2.4, maxDepth / 3.6]
+  if numberOfSteps <= 3:
+    return reversed(steps[:numberOfSteps])
+  if numberOfSteps > 3:
+    steps.extend((maxDepth / (3.6 + 0.2 * (numberOfSteps - i)) for i in reversed(range(numberOfSteps - 3))))
+  return reversed(steps)
+
+
+def __createComboBox(layout, status, values, info, names, widgets, nameIdx):
+  from PyQt5.QtWidgets import QComboBox, QGridLayout, QWidget
+
+  name, choices, callback, getCurrentIndex = info
+  combobox = QComboBox()
+  combobox.setContentsMargins(20, 5, 20, 5)
+  combobox.addItems(choices)
+  combobox.setCurrentIndex(getCurrentIndex(values))
+  listView = combobox.view()
+  listView.setWordWrap(True)
+  listView.adjustSize()
+  combobox.currentIndexChanged.connect(lambda idx: callback(idx, values))
+  combobox.currentIndexChanged.connect(lambda idx: widgets['loop'].exit())
+
+  layout = QGridLayout()  # this is required to align combobox with sliders
+  layout.setColumnStretch(0, 1)
+  layout.setRowStretch(0, 1)
+  layout.setColumnStretch(2, 1)
+  layout.setVerticalSpacing(0)
+  layout.setContentsMargins(20, 5, 20, 5)
+  layout.addWidget(combobox, 1, 1)
+  wrapperWidget = QWidget()
+  wrapperWidget.setLayout(layout)
+
+  return name, wrapperWidget
+
+
+def __createSlider(layout, status, values, info, names, widgets, hasCheckbox, nameIdx, widgetPosition=None):
+  from PyQt5.QtCore import QAbstractListModel
 
   import zebrazoom.code.util as util
 
-  minn, maxx, hint = info
-  double = name == "authorizedRelativeLengthTailEnd"
-  slider = util.SliderWithSpinbox(values[name], minn, maxx, name=name, double=double)
-  if name == "eyeFilterKernelSize":
+  stepIdx = None
+  if isinstance(names, QAbstractListModel):
+    stepsIdx = names.itemlist.index('steps')
+    stepCount = len(values['steps'])
+    names.updateSteps(stepCount)
+    if stepsIdx < nameIdx <= stepsIdx + stepCount:
+      stepIdx = nameIdx - (stepsIdx + 1)
+      nameIdx = stepsIdx
+      name = f'Step {stepIdx + 1}'
+    else:
+      name = names.itemlist[nameIdx]
+      if nameIdx > stepsIdx:
+        nameIdx -= stepCount
+    minn, maxx, hint = info[nameIdx]
+  else:
+    name = names
+    minn, maxx, hint = info
+  if name not in values:
+    assert name.startswith('Step')
+    value = values['steps'][stepIdx]
+    minn = 0.01
+    maxx = max(values['maxDepth'], value * 1.33)
+  else:
+    value = values[name] if name != 'steps' else len(values[name])
+    if value < minn:
+      minn = int(value)
+    if value > maxx:
+      maxx = int(value * 1.33)
+  double = name in ("authorizedRelativeLengthTailEnd", "authorizedRelativeLengthTailEnd2", "maxDepth", "thetaDiffAccept", "thetaDiffAcceptAfterAuthorizedRelativeLengthTailEnd", "thetaDiffAcceptAfterAuthorizedRelativeLengthTailEnd2") or name not in values
+  if isinstance(names, QAbstractListModel):
+    slider = util.SliderWithSpinbox(value, minn, maxx, name=name, double=double, choices=names)
+
+    def choiceChanged(idx):
+      if not slider.isVisible():
+        return
+      choiceIdx = idx // 2
+      if names.itemlist[choiceIdx] in widgets:
+        widget = widgets[names.itemlist[choiceIdx]]
+        blocked = widget.blockSignals(True)
+        widget.setChoice(idx)
+        widget.blockSignals(blocked)
+        layout.replaceWidget(slider, widget)
+        slider.hide()
+        widget.show()
+      else:
+        layout.removeWidget(slider)
+        slider.hide()
+        _createWidget(layout, status, values, info, names, widgets, hasCheckbox, nameIdx=choiceIdx, widgetPosition=widgetPosition)
+    slider.choiceChanged.connect(choiceChanged)
+  else:
+    slider = util.SliderWithSpinbox(value, minn, maxx, name=name, double=double)
+  if name == "eyeFilterKernelSize" or name == "paramGaussianBlur":
     slider.setSingleStep(2)
 
   def showHint(fn):
@@ -34,28 +118,64 @@ def _createWidget(layout, status, values, info, name, widgets, hasCheckbox):
   slider.leaveEvent = hideHint(slider.leaveEvent)
 
   def valueChanged():
-    values[name] = slider.value()
+    if name == 'paramGaussianBlur' and not slider.value() % 2:
+      slider.setValue(slider.value() + 1)
+      return
+    if name == 'paramGaussianBlurForHeadPosition' and slider.value() and not slider.value() % 2:
+      slider.setValue(slider.value() + 1)
+      return
+    if name not in values:
+      assert name.startswith('Step ')
+      values['steps'][stepIdx] = slider.value()
+    elif name == 'steps':
+      newSteps = slider.value()
+      currentSteps = len(values[name])
+      difference = newSteps - currentSteps
+      if difference > 0:
+        newValues = []
+        for idx in range(difference):
+          stepName = f'Step {idx + 1}'
+          defaultSteps = list(__calculateSteps(values['maxDepth'], newSteps))
+          newValues.append(defaultSteps[idx])
+        values[name][:0] = newValues
+      elif difference < 0:
+        del values[name][:-difference]
+      names.updateSteps(len(values[name]))
+    else:
+      values[name] = slider.value()
+    if name == 'headEmbededParamTailDescentPixThreshStop':
+      values['maximumMedianValueOfAllPointsAlongTheTail'] = slider.value()
+      values['minimumHeadPixelValue'] = slider.value()
     widgets['loop'].exit()
   slider.valueChanged.connect(valueChanged)
+  return name, slider
+
+def _createWidget(layout, status, values, info, names, widgets, hasCheckbox, nameIdx=0, widgetPosition=None):
+  from PyQt5.QtCore import Qt
+
+  if isinstance(names, tuple):
+    name, widget = __createComboBox(layout, status, values, info, names, widgets, nameIdx)
+  else:
+    name, widget = __createSlider(layout, status, values, info, names, widgets, hasCheckbox, nameIdx, widgetPosition=layout.count()if widgetPosition is None else widgetPosition)
 
   if name != "Frame number":
-    elements = layout.count() - (4 if hasCheckbox else 3)  # frame, status, checkbox, frameSlider
+    elements = (widgetPosition if widgetPosition is not None else layout.count()) - (4 if hasCheckbox else 3)  # frame, status, checkbox, frameSlider
     row = elements // 2 + (4 if hasCheckbox else 3)
     col = elements % 2
     if len(values) == 1:
-      layout.addWidget(slider, row, col, 1, 2, Qt.AlignmentFlag.AlignCenter)
+      layout.addWidget(widget, row, col, 1, 2, Qt.AlignmentFlag.AlignCenter)
     else:
-      layout.addWidget(slider, row, col, Qt.AlignmentFlag.AlignLeft if col else Qt.AlignmentFlag.AlignRight)
-    widgets[name] = slider
+      layout.addWidget(widget, row, col, Qt.AlignmentFlag.AlignLeft if col else Qt.AlignmentFlag.AlignRight)
+    widgets[name] = widget
   else:
-    widgets['frameSlider'] = slider
-    layout.addWidget(slider, 3 if hasCheckbox else 2, 0, 1, 2, Qt.AlignmentFlag.AlignCenter)
+    widgets['frameSlider'] = widget
+    layout.addWidget(widget, 3 if hasCheckbox else 2, 0, 1, 2, Qt.AlignmentFlag.AlignCenter)
 
 
-def adjustHyperparameters(l, hyperparameters, hyperparametersListNames, frameToShow, title, organizationTab, widgets, documentationLink=None, addContrastCheckbox=False):
-  from PyQt5.QtCore import Qt, QEventLoop, QTimer
+def adjustHyperparameters(l, hyperparameters, hyperparametersListNames, frameToShow, title, organizationTab, widgets, documentationLink=None, addContrastCheckbox=False, addZoomCheckbox=False, addUnprocessedFrameCheckbox=False):
+  from PyQt5.QtCore import Qt, QAbstractListModel, QEventLoop, QTimer
   from PyQt5.QtGui import QCursor
-  from PyQt5.QtWidgets import QApplication, QCheckBox, QGridLayout, QHBoxLayout, QLabel, QPushButton, QVBoxLayout, QWidget
+  from PyQt5.QtWidgets import QApplication, QCheckBox, QGridLayout, QHBoxLayout, QLabel, QPushButton, QSpinBox, QVBoxLayout, QWidget
 
   import zebrazoom.code.paths as paths
   import zebrazoom.code.util as util
@@ -64,6 +184,8 @@ def adjustHyperparameters(l, hyperparameters, hyperparametersListNames, frameToS
   stackedLayout = app.window.centralWidget().layout()
   timers = []
 
+  hasCheckbox = addContrastCheckbox or addZoomCheckbox or addUnprocessedFrameCheckbox
+
   if widgets is None:
     widgets = {'Frame number': l, 'saved': False, 'discarded': False, 'loop': QEventLoop()}
     layout = QGridLayout()
@@ -75,19 +197,41 @@ def adjustHyperparameters(l, hyperparameters, hyperparametersListNames, frameToS
     status = QLabel()
     layout.addWidget(status, 1, 0, 1, 2, Qt.AlignmentFlag.AlignCenter)
 
-    if addContrastCheckbox:
-      checkbox = QCheckBox('Improve contrast')
-      widgets['contrastCheckbox'] = checkbox
-      checkbox.toggled.connect(lambda: widgets['loop'].exit())
-      QTimer.singleShot(0, lambda: checkbox.setChecked(hyperparameters["outputValidationVideoContrastImprovement"]))
-      layout.addWidget(checkbox, 2, 0, 1, 2, Qt.AlignmentFlag.AlignCenter)
+    if hasCheckbox:
+      checkboxesLayout = QHBoxLayout()
+      checkboxesLayout.addStretch(1)
+      if addContrastCheckbox:
+        checkbox = QCheckBox('Improve contrast')
+        widgets['contrastCheckbox'] = checkbox
+        checkbox.toggled.connect(lambda: widgets['loop'].exit())
+        QTimer.singleShot(0, lambda: checkbox.setChecked(hyperparameters["outputValidationVideoContrastImprovement"]))
+        checkboxesLayout.addWidget(checkbox, alignment=Qt.AlignmentFlag.AlignCenter)
+      if addUnprocessedFrameCheckbox:
+        checkbox = QCheckBox('Show unprocessed frame')
+        widgets['unprocessedFrameCheckbox'] = checkbox
+        checkbox.toggled.connect(lambda: widgets['loop'].exit())
+        checkboxesLayout.addWidget(checkbox, alignment=Qt.AlignmentFlag.AlignCenter)
+      if addZoomCheckbox:
+        numberOfAnimals = hyperparameters.get('nbAnimalsPerWell', 1)
+        checkbox = QCheckBox('Zoom in to animal')
+        checkbox.getAnimalIdx = lambda: animalSpinbox.value() if numberOfAnimals > 1 else 0
+        widgets['zoomInCheckbox'] = checkbox
+        checkbox.toggled.connect(lambda: widgets['loop'].exit())
+        checkboxesLayout.addWidget(checkbox, alignment=Qt.AlignmentFlag.AlignCenter)
+        if numberOfAnimals > 1:
+          animalSpinbox = QSpinBox()
+          animalSpinbox.setRange(0, numberOfAnimals - 1)
+          animalSpinbox.valueChanged.connect(lambda: widgets['loop'].exit())
+          checkboxesLayout.addWidget(animalSpinbox, alignment=Qt.AlignmentFlag.AlignLeft)
+      checkboxesLayout.addStretch(1)
+      layout.addLayout(checkboxesLayout, 2, 0, 1, 2, Qt.AlignmentFlag.AlignCenter)
 
     if l is not None:
-      _createWidget(layout, status, widgets, (hyperparameters["firstFrame"], hyperparameters["lastFrame"] - 1, "You can also go through the video with the keys left arrow (backward); right arrow (forward); page down (fast backward); page up (fast forward)"), "Frame number", widgets, addContrastCheckbox)
+      _createWidget(layout, status, widgets, (hyperparameters["firstFrame"], hyperparameters["lastFrame"] - 1, "You can also go through the video with the keys left arrow (backward); right arrow (forward); page down (fast backward); page up (fast forward)"), "Frame number", widgets, hasCheckbox)
     else:
       layout.addWidget(QWidget())
     for info, name in zip(organizationTab, hyperparametersListNames):
-      _createWidget(layout, status, hyperparameters, info, name, widgets, addContrastCheckbox)
+      _createWidget(layout, status, hyperparameters, info, name, widgets, hasCheckbox)
 
     mainLayout = QVBoxLayout()
     titleLabel = util.apply_style(QLabel(title), font=util.TITLE_FONT)
@@ -123,20 +267,48 @@ def adjustHyperparameters(l, hyperparameters, hyperparametersListNames, frameToS
     frame.show()
     util.setPixmapFromCv(frameToShow, frame)
   else:
+    flattenedList = []
     for name in hyperparametersListNames:
+      if isinstance(name, QAbstractListModel):
+        flattenedList.extend((n for n in name.itemlist if n in hyperparameters))
+      else:
+        flattenedList.append(name)
+    if 'steps' in hyperparameters:
+      for idx in range(len(hyperparameters['steps'])):
+        name = f'Step {idx + 1}'
+        if name not in widgets:
+          continue
+        slider = widgets[name]
+        value = hyperparameters['steps'][idx]
+        if slider.value() != value:
+          slider.setValue(value)
+        minn = slider.minimum()
+        maxx = slider.maximum()
+        if (value - minn) > (maxx - minn) * 0.9:
+          maxx = int(minn + (value - minn) * 1.1)
+        elif maxx - minn > 255 and value < minn + (maxx - minn) * 0.1:
+          maxx = int(minn + (maxx - minn) * 0.9)
+        else:
+          continue
+        maxx = min(maxx, MAX_INT32)
+        slider.setMaximum(maxx)
+    for name in flattenedList:
+      if name not in widgets:
+        continue
       slider = widgets[name]
-      if slider.value() != hyperparameters[name]:
-        slider.setValue(hyperparameters[name])
-      if name == "authorizedRelativeLengthTailEnd" or name == "eyeBinaryThreshold":
+      value = len(hyperparameters[name]) if name == 'steps' else hyperparameters[name]
+      if slider.value() != value:
+        slider.setValue(value)
+      if name.startswith("authorizedRelativeLengthTailEnd") or name.startswith("thetaDiffAccept") or name == "eyeBinaryThreshold":
         continue
       minn = slider.minimum()
       maxx = slider.maximum()
       if name == "frameGapComparision" and maxx == hyperparameters["lastFrame"] - hyperparameters["firstFrame"] - 1:
         continue
-      if (hyperparameters[name] - minn) > (maxx - minn) * 0.9:
-        maxx = minn + (hyperparameters[name] - minn) * 1.1
-      elif maxx - minn > 255 and hyperparameters[name] < minn + (maxx - minn) * 0.1:
-        maxx = minn + (maxx - minn) * 0.9
+      if (value - minn) > (maxx - minn) * 0.9:
+        maxx = int(minn + (value - minn) * 1.1)
+      elif maxx - minn > 255 and value < minn + (maxx - minn) * 0.1:
+        maxx = int(minn + (maxx - minn) * 0.9)
       else:
         continue
       if name == "frameGapComparision":
@@ -151,7 +323,7 @@ def adjustHyperparameters(l, hyperparameters, hyperparametersListNames, frameToS
         timer = QTimer()
         timers.append(timer)
         timer.setSingleShot(True)
-        newPosition = min(localCursorX / slider.sliderWidth(), 1) * maxx
+        newPosition = int(min(localCursorX / slider.sliderWidth(), 1) * maxx)
         timer.timeout.connect(lambda slider=slider: (slider.setPosition(newPosition) if newPosition > 0 else widgets['loop'].exit()) or timers.remove(timer))
         timer.start(30)
     if 'frameSlider' in widgets:
@@ -166,7 +338,19 @@ def adjustHyperparameters(l, hyperparameters, hyperparametersListNames, frameToS
     timer.stop()
 
   if widgets['saved']:
-    pickle.dump({name: hyperparameters[name] for name in hyperparametersListNames}, open(os.path.join(paths.getRootDataFolder(), 'newhyperparameters'), 'wb'))
+    newParams = {}
+    for name in hyperparametersListNames:
+      if isinstance(name, QAbstractListModel):
+        for name in name.itemlist:
+          if name in hyperparameters:
+            newParams[name] = hyperparameters[name]
+      elif isinstance(name, tuple):
+        for n in name:
+          if hyperparameters.get(n, False):
+            newParams[n] = hyperparameters[n]
+      else:
+        newParams[name] = hyperparameters[name]
+    pickle.dump(newParams, open(os.path.join(paths.getRootDataFolder(), 'newhyperparameters'), 'wb'))
     temporaryPage = stackedLayout.currentWidget()
     stackedLayout.setCurrentWidget(widgets['oldWidget'])
     stackedLayout.removeWidget(temporaryPage)
