@@ -4,13 +4,19 @@ import pandas as pd
 import os
 import time
 import datetime
+import math
 from difflib import SequenceMatcher
 
-def calculateSleepVsMovingPeriods(pathToZZoutput, vidName, speedThresholdForMoving, notMovingNumberOfFramesThresholdForSleep, specifiedStartTime=0, distanceTravelledRollingMedianFilter=0, videoPixelSize=-1, videoFPS=-1):
+from zebrazoom.dataAPI._createSuperStructFromH5 import createSuperStructFromH5
+from zebrazoom.dataAPI._openResultsFile import openResultsFile
+
+
+def calculateSleepVsMovingPeriods(pathToZZoutput, vidName, speedThresholdForMoving, notMovingNumberOfFramesThresholdForSleep, maxDistBetweenTwoPointsInsideSleepingPeriod=-1, specifiedStartTime=0, distanceTravelledRollingMedianFilter=0, videoPixelSize=-1, videoFPS=-1):
   
   videoNamelist    = []
   dfFinalAllVideos = pd.DataFrame()
   currentNbFrames  = 0
+  startSleepPeriod = -1
   
   if ',' in vidName:
     videoNamelist = vidName.split(',')
@@ -18,12 +24,15 @@ def calculateSleepVsMovingPeriods(pathToZZoutput, vidName, speedThresholdForMovi
     videoNamelist = [vidName]
   
   for idx, videoName in enumerate(videoNamelist):
-  
-    pathToVideo = os.path.join(pathToZZoutput, videoName, "results_" + videoName + ".txt")
-    
-    with open(pathToVideo) as ff:
-      dataRef = json.load(ff)
-    
+
+    try:
+      with openResultsFile(os.path.join(pathToZZoutput, videoName), 'r') as results:
+        dataRef = createSuperStructFromH5(results)
+    except ValueError:  # h5 results not found, assume a results folder exists
+      pathToVideo = os.path.join(pathToZZoutput, videoName, "results_" + videoName + ".txt")
+      with open(pathToVideo) as ff:
+        dataRef = json.load(ff)
+
     if videoPixelSize == -1:
       if "videoPixelSize" in dataRef:
         videoPixelSize = dataRef["videoPixelSize"]
@@ -51,12 +60,14 @@ def calculateSleepVsMovingPeriods(pathToZZoutput, vidName, speedThresholdForMovi
     
     for wellNumber in range(len(dataRef['wellPoissMouv'])):
       
-      displacementVector = videoPixelSize * np.diff(np.array([dataRef['wellPoissMouv'][wellNumber][0][0]["HeadX"], dataRef['wellPoissMouv'][wellNumber][0][0]["HeadY"]]))
+      headPosition = np.array([dataRef['wellPoissMouv'][wellNumber][0][0]["HeadX"], dataRef['wellPoissMouv'][wellNumber][0][0]["HeadY"]])
+      displacementVector = videoPixelSize * np.diff(headPosition)
       squaredDisplacement = displacementVector * displacementVector
       distance = np.sqrt(squaredDisplacement[0] + squaredDisplacement[1])
       # distance = np.append(distance, [0])
       distance = np.insert(distance, 0, 0)
       distance = np.insert(distance, 0, 0)
+      headPosition = np.concatenate((np.zeros((2,1)), headPosition), axis=1)
       if distanceTravelledRollingMedianFilter:
         distance2 = np.convolve(distance, np.ones(distanceTravelledRollingMedianFilter), 'same') / distanceTravelledRollingMedianFilter
         distance2[:distanceTravelledRollingMedianFilter-1] = distance[:distanceTravelledRollingMedianFilter-1]
@@ -72,25 +83,34 @@ def calculateSleepVsMovingPeriods(pathToZZoutput, vidName, speedThresholdForMovi
       for i in range(len(sleep)):
         if moving[i]:
           countNotMovingFrames = 0
+          startSleepPeriod     = -1
         else:
-          countNotMovingFrames = countNotMovingFrames + 1
-          if countNotMovingFrames >= notMovingNumberOfFramesThresholdForSleep:
-            sleep[i] = True
-          if countNotMovingFrames == notMovingNumberOfFramesThresholdForSleep:
-            countNotMovingFrames2 = countNotMovingFrames - 1
-            j = i - 1
-            while j >= 0 and countNotMovingFrames2 > 0:
-              sleep[j] = True
-              countNotMovingFrames2 = countNotMovingFrames2 - 1
-              j = j - 1
+          if maxDistBetweenTwoPointsInsideSleepingPeriod == -1 or startSleepPeriod == -1 or (videoPixelSize*math.sqrt((headPosition[0][startSleepPeriod] - headPosition[0][i])**2 + (headPosition[1][startSleepPeriod] - headPosition[1][i])**2)) < maxDistBetweenTwoPointsInsideSleepingPeriod:
+            countNotMovingFrames = countNotMovingFrames + 1
+            if countNotMovingFrames == 1:
+              startSleepPeriod = i
+            if countNotMovingFrames >= notMovingNumberOfFramesThresholdForSleep:
+              sleep[i] = True
+            if countNotMovingFrames == notMovingNumberOfFramesThresholdForSleep:
+              countNotMovingFrames2 = countNotMovingFrames - 1
+              j = i - 1
+              while j >= 0 and countNotMovingFrames2 > 0:
+                sleep[j] = True
+                countNotMovingFrames2 = countNotMovingFrames2 - 1
+                j = j - 1
+          else:
+            countNotMovingFrames = 0
+            startSleepPeriod     = -1
       
       df = pd.DataFrame(data = np.transpose(np.append(np.append([speed], [moving], axis=0), [sleep], axis=0)), columns = ['speed_' + str(wellNumber), 'moving_' + str(wellNumber), 'sleep_' + str(wellNumber)])
       dfFinal = pd.concat([dfFinal, df], axis=1)
-    
-    dfFinal.to_excel(os.path.join(pathToZZoutput, videoName, "sleepVsMoving_" + videoName + ".xlsx"))
+
+    resultsFolder = os.path.join(pathToZZoutput, videoName)
+    if not os.path.exists(resultsFolder):
+      os.makedirs(resultsFolder)
+    dfFinal.to_excel(os.path.join(resultsFolder, "sleepVsMoving_" + videoName + ".xlsx"))
     dfFinalAllVideos = pd.concat([dfFinalAllVideos, dfFinal], axis=0)
     
-    
   if len(videoNamelist) > 1:
     name1 = videoNamelist[0]
     name2 = videoNamelist[1]
