@@ -5,14 +5,38 @@ import numpy as np
 
 import cv2
 
-from PyQt5.QtCore import pyqtSignal, Qt, QAbstractAnimation, QEventLoop, QLine, QParallelAnimationGroup, QPoint, QPointF, QPropertyAnimation, QRectF, QSize, QSizeF, QStandardPaths, QTimer
-from PyQt5.QtGui import QBrush, QColor, QFont, QImage, QPainter, QPen, QPixmap, QPolygonF, QTransform
-from PyQt5.QtWidgets import QApplication, QDoubleSpinBox, QFrame, QGraphicsPixmapItem, QGraphicsScene, QGraphicsView, QGridLayout, QLabel, QLayout, QHBoxLayout, QPushButton, QScrollArea, QSizePolicy, QSlider, QSpinBox, QSplitter, QSplitterHandle, QToolButton, QToolTip, QVBoxLayout, QWidget
+from PyQt5.QtCore import pyqtSignal, Qt, QAbstractAnimation, QAbstractEventDispatcher, QEventLoop, QLine, QParallelAnimationGroup, QPoint, QPointF, QPropertyAnimation, QRect, QRectF, QSize, QSizeF, QStandardPaths, QTimer
+from PyQt5.QtGui import QBrush, QColor, QFont, QImage, QPainter, QPen, QPixmap, QPolygon, QPolygonF, QTransform
+from PyQt5.QtWidgets import QApplication, QComboBox, QDoubleSpinBox, QFrame, QGraphicsPixmapItem, QGraphicsScene, QGraphicsView, QGridLayout, QLabel, QLayout, QHBoxLayout, QPushButton, QScrollArea, QSizePolicy, QSlider, QSpinBox, QSplitter, QSplitterHandle, QToolButton, QToolTip, QVBoxLayout, QWidget
 PYQT6 = False
 
 import zebrazoom.videoFormatConversion.zzVideoReading as zzVideoReading
 
 
+VIDEO_EXTENSIONS = {'.264', '.3g2', '.3gp', '.3gp2', '.3gpp', '.3gpp2', '.3mm', '.3p2', '.60d', '.787', '.89', '.aaf', '.aec', '.aep', '.aepx', '.aet', '.aetx', '.ajp',
+                    '.ale', '.am', '.amc', '.amv', '.amx', '.anim', '.aqt', '.arcut', '.arf', '.asf', '.asx', '.avb', '.avc', '.avd', '.avi', '.avp', '.avs', '.avs',
+                    '.avv', '.axm', '.bdm', '.bdmv', '.bdt2', '.bdt3', '.bik', '.bix', '.bmk', '.bnp', '.box', '.bs4', '.bsf', '.bvr', '.byu', '.camproj', '.camrec',
+                    '.camv', '.ced', '.cel', '.cine', '.cip', '.clpi', '.cmmp', '.cmmtpl', '.cmproj', '.cmrec', '.cpi', '.cst', '.cvc', '.cx3', '.d2v', '.d3v', '.dat',
+                    '.dav', '.dce', '.dck', '.dcr', '.dcr', '.ddat', '.dif', '.dir', '.divx', '.dlx', '.dmb', '.dmsd', '.dmsd3d', '.dmsm', '.dmsm3d', '.dmss', '.dmx', '.dnc',
+                    '.dpa', '.dpg', '.dream', '.dsy', '.dv', '.dv-avi', '.dv4', '.dvdmedia', '.dvr', '.dvr-ms', '.dvx', '.dxr', '.dzm', '.dzp', '.dzt', '.edl', '.evo', '.eye',
+                    '.ezt', '.f4p', '.f4v', '.fbr', '.fbr', '.fbz', '.fcp', '.fcproject', '.ffd', '.flc', '.flh', '.fli', '.flv', '.flx', '.gfp', '.gl', '.gom', '.grasp',
+                    '.gts', '.gvi', '.gvp', '.h264', '.hdmov', '.hkm', '.ifo', '.imovieproj', '.imovieproject', '.ircp', '.irf', '.ism', '.ismc', '.ismv', '.iva', '.ivf',
+                    '.ivr', '.ivs', '.izz', '.izzy', '.jss', '.jts', '.jtv', '.k3g', '.kmv', '.ktn', '.lrec', '.lsf', '.lsx', '.m15', '.m1pg', '.m1v', '.m21', '.m21', '.m2a',
+                    '.m2p', '.m2t', '.m2ts', '.m2v', '.m4e', '.m4u', '.m4v', '.m75', '.mani', '.meta', '.mgv', '.mj2', '.mjp', '.mjpg', '.mk3d', '.mkv', '.mmv', '.mnv',
+                    '.mob', '.mod', '.modd', '.moff', '.moi', '.moov', '.mov', '.movie', '.mp21', '.mp21', '.mp2v', '.mp4', '.mp4v', '.mpe', '.mpeg', '.mpeg1', '.mpeg4',
+                    '.mpf', '.mpg', '.mpg2', '.mpgindex', '.mpl', '.mpl', '.mpls', '.mpsub', '.mpv', '.mpv2', '.mqv', '.msdvd', '.mse', '.msh', '.mswmm', '.mts', '.mtv',
+                    '.mvb', '.mvc', '.mvd', '.mve', '.mvex', '.mvp', '.mvp', '.mvy', '.mxf', '.mxv', '.mys', '.ncor', '.nsv', '.nut', '.nuv', '.nvc', '.ogm', '.ogv', '.ogx',
+                    '.osp', '.otrkey', '.pac', '.par', '.pds', '.pgi', '.photoshow', '.piv', '.pjs', '.playlist', '.plproj', '.pmf', '.pmv', '.pns', '.ppj', '.prel', '.pro',
+                    '.prproj', '.prtl', '.psb', '.psh', '.pssd', '.pva', '.pvr', '.pxv', '.qt', '.qtch', '.qtindex', '.qtl', '.qtm', '.qtz', '.r3d', '.rcd', '.rcproject',
+                    '.rdb', '.rec', '.rm', '.rmd', '.rmd', '.rmp', '.rms', '.rmv', '.rmvb', '.roq', '.rp', '.rsx', '.rts', '.rts', '.rum', '.rv', '.rvid', '.rvl', '.sbk',
+                    '.sbt', '.scc', '.scm', '.scm', '.scn', '.screenflow', '.sec', '.sedprj', '.seq', '.sfd', '.sfvidcap', '.siv', '.smi', '.smi', '.smil', '.smk', '.sml',
+                    '.smv', '.spl', '.sqz', '.srt', '.ssf', '.ssm', '.stl', '.str', '.stx', '.svi', '.swf', '.swi', '.swt', '.tda3mt', '.tdx', '.thp', '.tivo', '.tix',
+                    '.tod', '.tp', '.tp0', '.tpd', '.tpr', '.trp', '.ts', '.tsp', '.ttxt', '.tvs', '.usf', '.usm', '.vc1', '.vcpf', '.vcr', '.vcv', '.vdo', '.vdr', '.vdx',
+                    '.veg','.vem', '.vep', '.vf', '.vft', '.vfw', '.vfz', '.vgz', '.vid', '.video', '.viewlet', '.viv', '.vivo', '.vlab', '.vob', '.vp3', '.vp6', '.vp7',
+                    '.vpj', '.vro', '.vs4', '.vse', '.vsp', '.w32', '.wcp', '.webm', '.wlmp', '.wm', '.wmd', '.wmmp', '.wmv', '.wmx', '.wot', '.wp3', '.wpl', '.wtv', '.wve',
+                    '.wvx', '.xej', '.xel', '.xesc', '.xfl', '.xlmv', '.xmv', '.xvid', '.y4m', '.yog', '.yuv', '.zeg', '.zm1', '.zm2', '.zm3', '.zmv', '.tif', 'tiff', '.bias'}
+
+
 PRETTY_PARAMETER_NAMES = {
   'frameGapComparision': 'Compare frame n with frame n+?',
   'thresForDetectMovementWithRawVideo': 'Threshold applied on frame n+? minus frame n',
@@ -22,6 +46,18 @@ PRETTY_PARAMETER_NAMES = {
   'overwriteFirstStepValue': 'Minimum number of pixels between subsequent points',
   'overwriteLastStepValue': 'Maximum number of pixels between subsequent points',
   'overwriteHeadEmbededParamGaussianBlur': 'Gaussian blur applied on the image',
+  'steps': 'Segment length between previous and next point: number of options',
+  'thetaDiffAccept': 'Maximum authorized angle difference between two subsequent segments in the first portion of the tail (in radians)',
+  'thetaDiffAcceptAfterAuthorizedRelativeLengthTailEnd': 'Maximum authorized angle difference between two subsequent segments in the second portion of the tail  (in radians)',
+  'thetaDiffAcceptAfterAuthorizedRelativeLengthTailEnd2': 'Maximum authorized angle difference between two subsequent segments in the third portion of the tail  (in radians)',
+  'nbList': 'Number of "candidates" points considered for next point along the tail in the first portion of the tail',
+  'nbListAfterAuthorizedRelativeLengthTailEnd': 'Number of "candidates" points considered for next point along the tail in the second portion of the tail',
+  'nbListAfterAuthorizedRelativeLengthTailEnd2': 'Number of "candidates" points considered for next point along the tail in the third portion of the tail',
+  'authorizedRelativeLengthTailEnd': 'Cut off relative location between first and second tail segment (between 0 and 1)',
+  'authorizedRelativeLengthTailEnd2': 'Cut off relative location between second and third tail segment (between 0 and 1)',
+  'maximumMedianValueOfAllPointsAlongTheTail': 'Maximum median pixel value of all points along the tail in order for the tail tracking to be accepted',
+  'minimumHeadPixelValue': 'Maximum pixel value authorized for a point to be considered as the head of the animal',
+  'nbTailPoints': 'Number of points along the tail in the output data',
 }
 
 TITLE_FONT = QFont('Helvetica', 18, QFont.Weight.Bold, True)
@@ -29,6 +65,7 @@ LIGHT_YELLOW = '#FFFFE0'
 LIGHT_CYAN = '#E0FFFF'
 LIGHT_GREEN = '#90ee90'
 GOLD = '#FFD700'
+DEFAULT_BUTTON_COLOR = LIGHT_GREEN
 SPINBOX_STYLESHEET = '''
 QSpinBox::down-button {
   subcontrol-origin: border;
@@ -94,17 +131,19 @@ def _getButtonsLayout(buttons, loop, dialog=None):
       loop.exit()
 
   for text, *args in buttons:
+    enabledSignal = None
+    color = None
     if len(args) == 1:
       cb, = args
       exitLoop = True
-      enabledSignal = None
     elif len(args) == 2:
       cb, exitLoop = args
-      enabledSignal = None
-    else:
-      assert len(args) == 3
+    elif len(args) == 3:
       cb, exitLoop, enabledSignal = args
-    button = QPushButton(text)
+    else:
+      assert len(args) == 4
+      cb, exitLoop, enabledSignal, color = args
+    button = QPushButton(text) if color is None else apply_style(QPushButton(text), background_color=color)
     if enabledSignal is not None:
       button.setEnabled(False)
       enabledSignal.connect(lambda enabled, btn=button: btn.setEnabled(bool(enabled)))
@@ -153,14 +192,37 @@ def showBlockingPage(layout, title=None, buttons=(), dialog=False, labelInfo=Non
   stackedLayout.removeWidget(temporaryPage)
 
 
-def showDialog(layout, title=None, buttons=(), labelInfo=None, timeout=None, exitSignals=()):
+def showInProgressPage(text):
+  def decorator(fn):
+    def inner(*args, **kwargs):
+      app = QApplication.instance()
+      layout = QVBoxLayout()
+      layout.addWidget(apply_style(QLabel("%s in progress..." % text), font=TITLE_FONT), alignment=Qt.AlignmentFlag.AlignCenter)
+      temporaryPage = QWidget()
+      temporaryPage.setLayout(layout)
+      stackedLayout = app.window.centralWidget().layout()
+      oldWidget = stackedLayout.currentWidget()
+      stackedLayout.addWidget(temporaryPage)
+      stackedLayout.setCurrentWidget(temporaryPage)
+      QAbstractEventDispatcher.instance().processEvents(QEventLoop.ProcessEventsFlag.AllEvents)  # XXX: this is ugly, but required to make sure the page is displayed before the function is executed, since the GUI will get blocked while the function is executing
+      try:
+        retval = fn(*args, **kwargs)  # XXX: GUI and non-GUI parts are still mixed so this function is expected to set the new page
+      finally:
+        stackedLayout.removeWidget(temporaryPage)
+      return retval
+    return inner
+  return decorator
+
+
+def showDialog(layout, title=None, buttons=(), labelInfo=None, timeout=None, exitSignals=(), focusWidgets=None):
   dialog = QWidget()
   for signal in exitSignals:
     signal.connect(lambda *args: dialog.close())
   loop = QEventLoop()
   mainLayout = QVBoxLayout()
   mainLayout.addLayout(layout)
-  mainLayout.addLayout(_getButtonsLayout(buttons, loop, dialog=dialog))
+  if buttons:
+    mainLayout.addLayout(_getButtonsLayout(buttons, loop, dialog=dialog))
   app = QApplication.instance()
   if hasattr(app, 'registerWindow'):
     app.registerWindow(dialog)
@@ -168,6 +230,9 @@ def showDialog(layout, title=None, buttons=(), labelInfo=None, timeout=None, exi
   dialog.move(0, 0)
   dialog.setWindowModality(Qt.WindowModality.ApplicationModal)
   dialog.setLayout(mainLayout)
+  screenSize = QApplication.primaryScreen().availableSize()
+  if hasattr(app, 'window'):
+    screenSize -= app.window.frameSize() - app.window.size()
   if labelInfo is not None:
     if len(labelInfo) == 2:
       img, label = labelInfo
@@ -178,14 +243,20 @@ def showDialog(layout, title=None, buttons=(), labelInfo=None, timeout=None, exi
     height, width = img.shape[:2]
     label.setMinimumSize(width, height)
     layoutSize = mainLayout.totalSizeHint()
-    label.setMinimumSize(1, 1)
+    widthDiff = layoutSize.width() - screenSize.width()
+    heightDiff = layoutSize.height() - screenSize.height()
+    if widthDiff > 0 or heightDiff > 0:
+      dialog.adjustSize()
+      labelSize = label.size()
+      label.setFixedSize(labelSize.scaled(labelSize - QSize(widthDiff if widthDiff > 0 else 0, heightDiff if heightDiff > 0 else 0), Qt.AspectRatioMode.KeepAspectRatio))
+      dialog.adjustSize()
+      layoutSize = mainLayout.totalSizeHint()
+      label.setMinimumSize(1, 1)
+      label.setMaximumSize(screenSize)
   else:
     layoutSize = mainLayout.totalSizeHint()
-  screenSize = QApplication.primaryScreen().availableSize()
-  if hasattr(app, 'window'):
-    screenSize -= app.window.frameSize() - app.window.size()
-  if layoutSize.width() > screenSize.width() or layoutSize.height() > screenSize.height():
-    layoutSize.scale(screenSize, Qt.AspectRatioMode.KeepAspectRatio)
+    if layoutSize.width() > screenSize.width() or layoutSize.height() > screenSize.height():
+      layoutSize.scale(screenSize, Qt.AspectRatioMode.KeepAspectRatio)
   dialog.setFixedSize(layoutSize)
   dialog.show()
   if labelInfo is not None:
@@ -199,6 +270,11 @@ def showDialog(layout, title=None, buttons=(), labelInfo=None, timeout=None, exi
   dialog.closeEvent = _dialogClosed(loop, dialog.closeEvent)
   if timeout is not None:
     QTimer.singleShot(timeout, lambda: dialog.close())
+  if focusWidgets is not None:
+    for child in dialog.findChildren(QWidget):
+      if child in focusWidgets or any(child.isAncestorOf(widget) for widget in focusWidgets) or any(widget.isAncestorOf(child) for widget in focusWidgets):
+        continue
+      child.setFocusPolicy(Qt.FocusPolicy.NoFocus)
   loop.exec()
 
 def _cvToPixmap(img):
@@ -393,6 +469,8 @@ class _ZoomableImageCircle(_ZoomableImage):
 
 
 def setPixmapFromCv(img, label, preferredSize=None, zoomable=False):
+  if img is None:
+    img = np.zeros((1, 1, 3), np.uint8)
   originalPixmap = _cvToPixmap(img)
   if not label.isVisible():
     label.setPixmap(originalPixmap)
@@ -403,8 +481,12 @@ def setPixmapFromCv(img, label, preferredSize=None, zoomable=False):
     label.setPixmap(originalPixmap)
     label.show()
   if preferredSize is None:
-    preferredSize = label.pixmap().size()
-  size = preferredSize.scaled(label.size(), Qt.AspectRatioMode.KeepAspectRatio)
+    preferredSize = originalPixmap.size()
+  labelSize = label.size()
+  if preferredSize.height() > labelSize.height() or preferredSize.width() > labelSize.width():
+    size = preferredSize.scaled(labelSize, Qt.AspectRatioMode.KeepAspectRatio)
+  else:
+    size = preferredSize
   if not zoomable:
     img = cv2.resize(img, (int(size.width() * scaling), int(size.height() * scaling)))
     pixmap = _cvToPixmap(img)
@@ -437,6 +519,89 @@ def setPixmapFromCv(img, label, preferredSize=None, zoomable=False):
       label.getInfo = calculateRadius
 
 
+class _TemporaryZoomableImagePoint(_ZoomableImage):
+  pointSelected = pyqtSignal(QPoint)
+  abort = pyqtSignal()
+
+  def __init__(self, parent=None, basePoint=None):
+    super().__init__(parent)
+    self._point = None
+    self._basePoint = None if basePoint is None else QPoint(*map(int, basePoint))
+    self._currentPosition = None
+    self.setMouseTracking(True)
+
+  def keyPressEvent(self, evt):
+    if evt.key() == Qt.Key.Key_Escape:
+      self.abort.emit()
+      return
+    super().keyPressEvent(evt)
+
+  def mouseMoveEvent(self, evt):
+    self._currentPosition = self.mapToScene(evt.pos()).toPoint()
+    self.viewport().update()
+    super().mouseMoveEvent(evt)
+
+  def mouseReleaseEvent(self, evt):
+    if not self._dragging:
+      if self._pixmap.isUnderMouse():
+        self._point = self.mapToScene(evt.pos()).toPoint()
+        self.viewport().update()
+        self.pointSelected.emit(self._point)
+    else:
+      self._dragging = False
+      QApplication.restoreOverrideCursor()
+    super().mouseReleaseEvent(evt)
+
+  def leaveEvent(self, evt):
+    QApplication.restoreOverrideCursor()
+    self._currentPosition = None
+    self.viewport().update()
+    super().leaveEvent(evt)
+
+  def paintEvent(self, evt):
+    super().paintEvent(evt)
+    if self._basePoint is None or self._currentPosition is None:
+      return
+    qp = QPainter(self.viewport())
+    qp.setPen(QColor(255, 0, 0))
+    qp.drawLine(self.mapFromScene(self._basePoint), self.mapFromScene(QPointF(self._currentPosition)))
+    qp.end()
+
+  def getCoordinates(self):
+    return None if self._point is None else (self._point.x(), self._point.y()) if self._basePoint is None else ((self._basePoint.x(), self._basePoint.y()), (self._point.x(), self._point.y()))
+
+
+def getPointOnFrame(frame, label, basePoint=None, exitSignals=()):
+  label.hide()
+  image = _TemporaryZoomableImagePoint(basePoint=basePoint)
+  labelSize = label.size()
+  pixmap = _cvToPixmap(frame)
+  size = pixmap.size()
+  if size.height() > labelSize.height() or size.width() > labelSize.width():
+    size = size.scaled(labelSize, Qt.AspectRatioMode.KeepAspectRatio)
+  widthDiff = labelSize.width() - size.width()
+  heightDiff = labelSize.height() - size.height()
+  horizontalMargin = widthDiff // 2
+  verticalMargin = heightDiff // 2
+  image.setViewportMargins(horizontalMargin + widthDiff % 2, verticalMargin + heightDiff % 2, horizontalMargin, verticalMargin)
+  image.sizeHint = lambda: labelSize
+  image.setMaximumSize(labelSize)
+  image.viewport().setFixedSize(size)
+  image.setPixmap(pixmap)
+  label.parentWidget().layout().replaceWidget(label, image)
+  image.setFocus()
+  loop = QEventLoop()
+  for signal in exitSignals:
+    signal.connect(lambda *args: loop.exit())
+  image.pointSelected.connect(lambda point: loop.exit())
+  image.abort.connect(loop.exit)
+  loop.exec()
+  image.hide()
+  image.parentWidget().layout().replaceWidget(image, label)
+  label.show()
+  return image.getCoordinates()
+
+
 class _DoubleSlider(QSlider):
   _DECIMALS = 2
 
@@ -462,11 +627,16 @@ class _DoubleSlider(QSlider):
 
 class SliderWithSpinbox(QWidget):
   valueChanged = pyqtSignal(int)
+  choiceChanged = pyqtSignal(int)
 
-  def __init__(self, value, minimum, maximum, name=None, double=False):
+  def __init__(self, value, minimum, maximum, name=None, double=False, choices=None):
     super().__init__()
-    minimum = int(minimum)
-    maximum = int(maximum)
+    type_ = int if not double else float
+    value = type_(value)
+    minimum = type_(minimum)
+    maximum = type_(maximum)
+    formatMin = lambda x: (math.floor(x) if type_ is float else int(x))
+    formatMax = lambda x: (math.ceil(x) if type_ is float else int(x))
 
     layout = QGridLayout()
     layout.setRowStretch(0, 1)
@@ -475,9 +645,9 @@ class SliderWithSpinbox(QWidget):
     layout.setColumnStretch(5, 1)
     layout.setVerticalSpacing(0)
 
-    minLabel = QLabel(str(minimum))
+    minLabel = QLabel(str(formatMin(minimum)))
     layout.addWidget(minLabel, 1, 1, Qt.AlignmentFlag.AlignLeft)
-    maxLabel = QLabel(str(maximum))
+    maxLabel = QLabel(str(formatMax(maximum)))
     layout.addWidget(maxLabel, 1, 3, Qt.AlignmentFlag.AlignRight)
     slider = QSlider(Qt.Orientation.Horizontal) if not double else _DoubleSlider(Qt.Orientation.Horizontal)
     slider.setMinimumWidth(350)
@@ -486,6 +656,8 @@ class SliderWithSpinbox(QWidget):
     layout.addWidget(slider, 2, 1, 1, 3)
 
     spinbox = QSpinBox() if not double else QDoubleSpinBox()
+    if double:
+      spinbox.setSingleStep(0.1)
     spinbox.setStyleSheet(SPINBOX_STYLESHEET)
     spinbox.setMinimumWidth(90)
     spinbox.setRange(minimum, maximum)
@@ -506,23 +678,35 @@ class SliderWithSpinbox(QWidget):
 
     if name is not None:
       self.setFixedWidth(layout.totalSizeHint().width())
-      titleLabel = QLabel(PRETTY_PARAMETER_NAMES.get(name, name))
-      titleLabel.setMinimumSize(1, 1)
-      titleLabel.resizeEvent = lambda evt: titleLabel.setMinimumWidth(evt.size().width()) or titleLabel.setWordWrap(evt.size().width() <= titleLabel.sizeHint().width())
-      slider.rangeChanged.connect(lambda: titleLabel.setMinimumWidth(1) or titleLabel.setWordWrap(False))
+      if choices is not None:
+        currentIdx = choices.itemlist.index(name) * 2
+        titleLabel = QComboBox()
+        titleLabel.setMaxVisibleItems(titleLabel.maxVisibleItems() * 2)
+        titleLabel.setModel(choices)
+        titleLabel.setCurrentIndex(currentIdx)
+        listView = titleLabel.view()
+        listView.setWordWrap(True)
+        listView.adjustSize()
+        titleLabel.currentIndexChanged.connect(lambda idx: self.choiceChanged.emit(idx))
+      else:
+        titleLabel = QLabel(PRETTY_PARAMETER_NAMES.get(name, name))
+        titleLabel.setMinimumSize(1, 1)
+        titleLabel.resizeEvent = lambda evt: titleLabel.setMinimumWidth(evt.size().width()) or titleLabel.setWordWrap(evt.size().width() <= titleLabel.sizeHint().width())
+        slider.rangeChanged.connect(lambda: titleLabel.setMinimumWidth(1) or titleLabel.setWordWrap(False))
       layout.addWidget(titleLabel, 1, 2, Qt.AlignmentFlag.AlignCenter)
 
     self.value = spinbox.value
     self.minimum = spinbox.minimum
     self.maximum = spinbox.maximum
     self.setValue = lambda value: spinbox.setValue(value) or slider.setValue(value)
-    self.setMinimum = lambda min_: spinbox.setMinimum(min_) or slider.setMinimum(min_) or minLabel.setText(str(int(min_)))
-    self.setMaximum = lambda max_: spinbox.setMaximum(max_) or slider.setMaximum(max_) or maxLabel.setText(str(int(max_)))
-    self.setRange = lambda min_, max_: spinbox.setRange(min_, max_) or slider.setRange(min_, max_)
+    self.setMinimum = lambda min_: spinbox.setMinimum(type_(min_)) or slider.setMinimum(type_(min_)) or minLabel.setText(str(formatMin(min_)))
+    self.setMaximum = lambda max_: spinbox.setMaximum(type_(max_)) or slider.setMaximum(type_(max_)) or maxLabel.setText(str(formatMax(max_)))
+    self.setRange = lambda min_, max_: spinbox.setRange(type_(min_), type_(max_)) or slider.setRange(type_(min_), type_(max_))
     self.setSingleStep = lambda step: spinbox.setSingleStep(step) or slider.setSingleStep(step)
     self.isSliderDown = slider.isSliderDown
     self.sliderWidth = slider.width
     self.setPosition = slider.setSliderPosition
+    self.setChoice = lambda idx: titleLabel.setCurrentIndex(idx)
 
 
 def _chooseFrameLayout(cap, spinboxValues, title, titleStyle=None):
@@ -594,7 +778,25 @@ def _chooseFrameLayout(cap, spinboxValues, title, titleStyle=None):
 
   return layout, video, frameSlider
 
-def chooseBeginningPage(app, videoPath, title, chooseFrameBtnText, chooseFrameBtnCb, extraButtonInfo=None, titleStyle=None, additionalLayout=None):
+
+def chooseFrame(app, videoPath, title, chooseFrameBtnText):
+  cap = zzVideoReading.VideoCapture(videoPath)
+  cap.set(1, 1)
+  ret, frame = cap.read()
+  layout, label, valueWidget = _chooseFrameLayout(cap, (1, 0, cap.get(7) - 2), title)
+
+  frameIdx = None
+  def chooseFrameBtnClicked():
+    nonlocal frameIdx
+    frameIdx = valueWidget.value()
+  showBlockingPage(layout, buttons=(("Back", lambda: None, True), ("Next", chooseFrameBtnClicked, True, None, DEFAULT_BUTTON_COLOR)), labelInfo=(frame, label, False))
+  if frameIdx is None:
+    return None
+  cap.set(1, frameIdx)
+  return cap.read()[1]
+
+
+def chooseBeginningPage(app, videoPath, title, chooseFrameBtnText, chooseFrameBtnCb, extraButtonInfo=None, titleStyle=None, additionalLayout=None, leftButtonInfo=None):
   cap = zzVideoReading.VideoCapture(videoPath)
   cap.set(1, 1)
   ret, frame = cap.read()
@@ -604,13 +806,19 @@ def chooseBeginningPage(app, videoPath, title, chooseFrameBtnText, chooseFrameBt
 
   buttonsLayout = QHBoxLayout()
   buttonsLayout.addStretch()
+  if leftButtonInfo is not None:
+    text, cb = leftButtonInfo
+    leftBtn = QPushButton(text)
+    leftBtn.clicked.connect(cb)
+    buttonsLayout.addWidget(leftBtn)
   if app.configFileHistory:
     backBtn = QPushButton("Back")
     backBtn.setObjectName("back")
     buttonsLayout.addWidget(backBtn)
-  chooseFrameBtn = QPushButton(chooseFrameBtnText)
+  chooseFrameBtn = QPushButton(chooseFrameBtnText) if extraButtonInfo is not None else apply_style(QPushButton(chooseFrameBtnText), background_color=DEFAULT_BUTTON_COLOR)
   def chooseFrameBtnClicked():
     app.configFile["firstFrame"] = valueWidget.value()
+    app.configFile["firstFrameForBackExtract"] = valueWidget.value()
     chooseFrameBtnCb()
   chooseFrameBtn.clicked.connect(chooseFrameBtnClicked)
   buttonsLayout.addWidget(chooseFrameBtn)
@@ -647,22 +855,32 @@ def chooseBeginningPage(app, videoPath, title, chooseFrameBtnText, chooseFrameBt
     btn.clicked.connect(lambda: stackedLayout.removeWidget(page))
 
 
-def chooseEndPage(app, videoPath, title, chooseFrameBtnText, chooseFrameBtnCb):
+def chooseEndPage(app, videoPath, title, chooseFrameBtnText, chooseFrameBtnCb, leftButtonInfo=None):
   cap = zzVideoReading.VideoCapture(videoPath)
   maximum = cap.get(7) - 2
-  cap.set(1, maximum)
-  ret, frame = cap.read()
+  while True:
+    cap.set(1, maximum)
+    ret, frame = cap.read()
+    if ret:
+      break
+    maximum -= 1
   layout, label, valueWidget = _chooseFrameLayout(cap, (maximum, app.configFile["firstFrame"] + 1, maximum), title)
 
   buttonsLayout = QHBoxLayout()
   buttonsLayout.addStretch()
+  if leftButtonInfo is not None:
+    text, cb = leftButtonInfo
+    leftBtn = QPushButton(text)
+    leftBtn.clicked.connect(cb)
+    buttonsLayout.addWidget(leftBtn)
   if app.configFileHistory:
     backBtn = QPushButton("Back")
     backBtn.setObjectName("back")
     buttonsLayout.addWidget(backBtn)
-  chooseFrameBtn = QPushButton(chooseFrameBtnText)
+  chooseFrameBtn = apply_style(QPushButton(chooseFrameBtnText), background_color=DEFAULT_BUTTON_COLOR)
   def chooseFrameBtnClicked():
     app.configFile["lastFrame"] = valueWidget.value()
+    app.configFile["lastFrameForBackExtract"] = valueWidget.value()
     chooseFrameBtnCb()
   chooseFrameBtn.clicked.connect(chooseFrameBtnClicked)
   buttonsLayout.addWidget(chooseFrameBtn)
@@ -775,16 +993,24 @@ def getPoint(frame, title, extraButtons=(), selectingRegion=False, backBtnCb=Non
       zoomableImage.hide()
       video.show()
     cb(video)
-  extraButtons = tuple((text, lambda: callback(cb), exitLoop) for text, cb, exitLoop in extraButtons)
+
+  def updateButton(args):
+    text, cb, exitLoop = args[:3]
+    retval = (text, lambda: callback(cb), exitLoop)
+    if len(args) == 4:
+      retval += (video.pointSelected,)
+    return retval
+
+  extraButtons = tuple(updateButton(button) for button in extraButtons)
   buttons = (("Back", backBtnCb, True),) if backBtnCb is not None else ()
-  buttons += (("Next", None, True, video.pointSelected),) if useNext else ()
+  buttons += (("Next", None, True, video.pointSelected, DEFAULT_BUTTON_COLOR),) if useNext else ()
   layout.addWidget(video, alignment=Qt.AlignmentFlag.AlignCenter, stretch=1)
   if not useNext:
     video.pointSelected.connect(lambda: QApplication.restoreOverrideCursor())
   if not dialog:
-    showBlockingPage(layout, title=title, buttons=buttons + extraButtons, labelInfo=(frame, video, zoomable), exitSignals=(video.proceed,) if useNext else (video.pointSelected,))
+    showBlockingPage(layout, title=title, buttons=extraButtons + buttons, labelInfo=(frame, video, zoomable), exitSignals=(video.proceed,) if useNext else (video.pointSelected,))
   else:
-    showDialog(layout, title=title, buttons=buttons + extraButtons, labelInfo=(frame, video, zoomable), exitSignals=(video.proceed,) if useNext else (video.pointSelected,))
+    showDialog(layout, title=title, buttons=extraButtons + buttons, labelInfo=(frame, video, zoomable), exitSignals=(video.proceed,) if useNext else (video.pointSelected,))
   return video.getCoordinates()
 
 
@@ -903,9 +1129,9 @@ def getRectangle(frame, title, backBtnCb=None, dialog=False, buttons=None, initi
     video.regionSelected.connect(clearRectangleBtn.setEnabled)
   if buttons is None:
     if backBtnCb is not None:
-      buttons = (("Back", backBtnCb, True), ("Next", None, True, video.regionSelected))
+      buttons = (("Back", backBtnCb, True), ("Next", None, True, video.regionSelected, DEFAULT_BUTTON_COLOR))
     else:
-      buttons = (("Next", None, True, video.regionSelected),)
+      buttons = (("Next", None, True, video.regionSelected, DEFAULT_BUTTON_COLOR),)
   else:
     assert backBtnCb is None
   if not dialog:
@@ -1071,9 +1297,12 @@ def addToHistory(fn):
 
 
 class Expander(QWidget):
-  def __init__(self, parent, title, layout, animationDuration=200, showFrame=False, addScrollbars=False):
+  expanded = pyqtSignal(bool)
+
+  def __init__(self, parent, title, layout, animationDuration=200, showFrame=False, addScrollbars=False, retainWidth=False):
     super().__init__()
 
+    self._retainWidth = retainWidth
     self._toggleButton = toggleButton = QToolButton()
     toggleButton.setStyleSheet("QToolButton { border: none; }")
     toggleButton.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
@@ -1120,6 +1349,7 @@ class Expander(QWidget):
     contentAnimation.setEndValue(contentHeight)
 
     def startAnimation(checked):
+      self.expanded.emit(checked)
       arrowType = Qt.ArrowType.DownArrow if checked else Qt.ArrowType.RightArrow
       direction = QAbstractAnimation.Direction.Forward if checked else QAbstractAnimation.Direction.Backward
       toggleButton.setArrowType(arrowType)
@@ -1127,6 +1357,14 @@ class Expander(QWidget):
       toggleAnimation.start()
     toggleButton.clicked.connect(startAnimation)
 
+  def updateLayout(self, layout):
+    widget = self._contentArea.widget()
+    if widget is None:
+      return
+    widget = QWidget(self)
+    widget.setLayout(layout)
+    self._contentArea.setWidget(widget)
+
   def refresh(self, availableHeight):
     layout = self._contentArea.layout()
     height = layout.sizeHint().height() if layout is not None else self._contentArea.widget().sizeHint().height() + 5
@@ -1139,6 +1377,16 @@ class Expander(QWidget):
       self._toggleAnimation.animationAt(i).setEndValue(self._collapseHeight + contentHeight)
     self._toggleAnimation.animationAt(self._toggleAnimation.animationCount() - 1).setEndValue(contentHeight)
 
+  def sizeHint(self):
+    hint = super().sizeHint()
+    if not self._retainWidth or self._contentArea.layout() is None:
+      return hint
+    contentHint = self._contentArea.layout().sizeHint()
+    return QSize(max(hint.width(), contentHint.width()), hint.height())
+
+  def isExpanded(self):
+    return self._toggleButton.isChecked()
+
 
 class _InteractiveLabelCircle(QLabel):
   circleSelected = pyqtSignal(bool)
@@ -1224,9 +1472,9 @@ def getCircle(frame, title, backBtnCb=None, zoomable=False):
   video = _InteractiveLabelCircle(width, height)
   layout.addWidget(video, alignment=Qt.AlignmentFlag.AlignCenter, stretch=1)
   if backBtnCb is not None:
-    buttons = (("Cancel", backBtnCb, True), ("Ok", None, True, video.circleSelected))
+    buttons = (("Cancel", backBtnCb, True), ("Ok", None, True, video.circleSelected, DEFAULT_BUTTON_COLOR))
   else:
-    buttons = (("Ok", None, True, video.circleSelected),)
+    buttons = (("Ok", None, True, video.circleSelected, DEFAULT_BUTTON_COLOR),)
   showBlockingPage(layout, title=title, buttons=buttons, labelInfo=(frame, video, zoomable))
   return video.getInfo()
 
@@ -1304,3 +1552,158 @@ def improveContrast(frame, quartile):
   if frameIsBGR:
     frame = cv2.cvtColor(frame, cv2.COLOR_GRAY2BGR)
   return frame
+
+
+class _RotationAngleLabel(QLabel):
+  def __init__(self):
+    super().__init__()
+    self._currentPosition = None
+    self.setMouseTracking(True)
+
+  def mouseMoveEvent(self, evt):
+    self._currentPosition = evt.pos()
+    self.update()
+
+  def enterEvent(self, evt):
+    QApplication.setOverrideCursor(Qt.CursorShape.CrossCursor)
+
+  def leaveEvent(self, evt):
+    QApplication.restoreOverrideCursor()
+    self._currentPosition = None
+    self.update()
+
+  def paintEvent(self, evt):
+    super().paintEvent(evt)
+    if self._currentPosition is None:
+      return
+    qp = QPainter()
+    qp.begin(self)
+    qp.setPen(QColor(255, 0, 0))
+    qp.drawLine(0, self._currentPosition.y(), self.width(), self._currentPosition.y())
+    qp.drawLine(self._currentPosition.x(), 0, self._currentPosition.x(), self.height())
+    qp.end()
+
+
+def getRotationAngle(frame, angle, dialog=False):
+  layout = QVBoxLayout()
+  video = _RotationAngleLabel()
+  layout.addWidget(video, alignment=Qt.AlignmentFlag.AlignCenter, stretch=1)
+
+  frameSlider = SliderWithSpinbox(angle, -180, 180, name="Rotation angle (degrees)", double=True)
+
+  def getFrame():
+    nonlocal angle
+    angle = frameSlider.value()
+    return cv2.warpAffine(frame, cv2.getRotationMatrix2D(tuple(x / 2 for x in frame.shape[1::-1]), angle, 1.0), frame.shape[1::-1], flags=cv2.INTER_LINEAR)
+  frameSlider.valueChanged.connect(lambda: setPixmapFromCv(getFrame(), video))
+  layout.addWidget(frameSlider, alignment=Qt.AlignmentFlag.AlignCenter)
+
+  cancelled = False
+  def cancel():
+    nonlocal cancelled
+    cancelled = True
+  buttons = (("Cancel", cancel, True), ("Ok", None, True))
+  if not dialog:
+    showBlockingPage(layout, title="Select the video rotation angle", buttons=buttons, labelInfo=(getFrame(), video), dialog=dialog)
+  else:
+    showDialog(layout, title="Select the video rotation angle", buttons=buttons, labelInfo=(getFrame(), video))
+  return None if cancelled else angle
+
+
+class WellSelectionLabel(QLabel):
+  def __init__(self, width, height, selectedWell=0):
+    super().__init__()
+    self._width = width
+    self._height = height
+    self._well = selectedWell
+    self._hoveredWell = None
+    if QApplication.instance().wellPositions:
+      self.setMouseTracking(True)
+
+  def mouseMoveEvent(self, evt):
+    app = QApplication.instance()
+    if not app.wellPositions:
+      return
+    oldHovered = self._hoveredWell
+    if app.wellShape == 'rectangle':
+      def test_func(point, x, y, width, height):
+        return QRect(x, y, width, height).contains(point)
+    else:
+      assert app.wellShape == 'circle'
+      def test_func(point, x, y, width, height):
+        radius = width / 2;
+        centerX = x + radius;
+        centerY = y + radius;
+        dx = abs(point.x() - centerX)
+        if dx > radius:
+          return False
+        dy = abs(point.y() - centerY)
+        if dy > radius:
+          return False
+        if dx + dy <= radius:
+          return True
+        return dx * dx + dy * dy <= radius * radius
+    for idx, positions in enumerate(app.wellPositions):
+      if test_func(self._transformToOriginal.map(evt.pos()), *positions):
+        self._hoveredWell = idx
+        break
+    else:
+      self._hoveredWell = None
+    if self._hoveredWell != oldHovered:
+      self.update()
+
+  def mousePressEvent(self, evt):
+    if not QApplication.instance().wellPositions:
+      return
+    if self._hoveredWell is not None:
+      self._well = self._hoveredWell
+      self.update()
+
+  def enterEvent(self, evt):
+    QApplication.setOverrideCursor(Qt.CursorShape.CrossCursor)
+
+  def leaveEvent(self, evt):
+    QApplication.restoreOverrideCursor()
+    self._hoveredWell = None
+    self.update()
+
+  def paintEvent(self, evt):
+    super().paintEvent(evt)
+    app = QApplication.instance()
+    if not app.wellPositions:
+      return
+    qp = QPainter()
+    qp.begin(self)
+    factory = qp.drawRect if app.wellShape == 'rectangle' else qp.drawEllipse
+    font = QFont()
+    font.setPointSize(16)
+    font.setWeight(QFont.Weight.Bold)
+    qp.setFont(font)
+    for idx, positions in enumerate(app.wellPositions):
+      if idx == self._well:
+        qp.setPen(QColor(255, 0, 0))
+      elif idx == self._hoveredWell:
+        qp.setPen(QColor(0, 255, 0))
+      else:
+        qp.setPen(QColor(0, 0, 255))
+      rect = self._transformFromOriginal.map(QPolygon(QRect(*positions))).boundingRect()
+      qp.drawText(rect, Qt.AlignmentFlag.AlignCenter, str(idx))
+      factory(rect)
+    qp.end()
+
+  def resizeEvent(self, evt):
+    super().resizeEvent(evt)
+    self._size = self.size()
+    originalRect = QRectF(QPointF(0, 0), QSizeF(self._width, self._height))
+    currentRect = QRectF(QPointF(0, 0), QSizeF(self._size))
+    self._transformToOriginal = QTransform()
+    QTransform.quadToQuad(QPolygonF((currentRect.topLeft(), currentRect.topRight(), currentRect.bottomLeft(), currentRect.bottomRight())),
+                          QPolygonF((originalRect.topLeft(), originalRect.topRight(), originalRect.bottomLeft(), originalRect.bottomRight())),
+                          self._transformToOriginal)
+    self._transformFromOriginal = QTransform()
+    QTransform.quadToQuad(QPolygonF((originalRect.topLeft(), originalRect.topRight(), originalRect.bottomLeft(), originalRect.bottomRight())),
+                          QPolygonF((currentRect.topLeft(), currentRect.topRight(), currentRect.bottomLeft(), currentRect.bottomRight())),
+                          self._transformFromOriginal)
+
+  def getWell(self):
+    return self._well
